package com.eclipse.listener.helpClasses;

import java.util.ArrayList;
import org.eclipse.swt.widgets.*;

/**
 * This java class is a help class for iterating through a SWT Shell object and add every children to an ArrayList<Control>.
 * This will make it easier to iterate through the ArrayList and get the Controls that is inside the Shell object.
 */
public class ShellObjectIterator {
	/**
	 * An ArrayList of the Shell-objects children Control-objects.
	 */
	private ArrayList<Control> controls = null;
	/**
	 * The Shell-object that is given as a parameter to the constructor method.
	 */
	private Shell shell = null;
	
	/**
	 * Constructor which takes a Shell-object as a parameter.
	 * @param Shell
	 */
	public ShellObjectIterator(Shell shell){
		this.shell = shell;		
		//Get the children of the Shell.
		if( getShell() != null ){
			System.out.println("[ShellObjectIterator]: Shell: " +getShell()+ ", name: " +getShell().getText());
			Control[] shellControls = getShell().getChildren();
			if( shellControls != null ){
				//Initialize ArrayList if the Shell has children Control-objects.
				controls = new ArrayList<Control>();
				System.out.println("[ShellObjectIterator]: Shell: " +getShell()+ ", number of children controls: " +shellControls.length);
				//Iterate through the Control-objects, adding them to the ArrayList, and recursively iterate through any Composite-instances.
				for(int i=0; i < shellControls.length; i++) {
					if( shellControls[i] instanceof Composite ){
						//Recursively iterate through Composite.
						iterateComposite( (Composite)shellControls[i] );
					}else{
						//Add Control-object to ArrayList.
						getControls().add( shellControls[i] );
						System.out.println("[ShellObjectIterator]: shellControls["+i+"]: " +shellControls[i]+ ", is not an instance of Composite");
					}
				}//end for.
			}
		}
	}//end ShellObjectIterator.
	
	/**
	 * Constructor which takes a Composite-object as a parameter.
	 * @param Shell
	 */
	public ShellObjectIterator(Composite composite){			
		//Get the children of the Composite.
		if( composite != null ){
			System.out.println("[ShellObjectIterator]: Composite: " +composite);
			Control[] compositeControls = composite.getChildren();
			if( compositeControls != null ){
				//Initialize ArrayList if the Composite has children Control-objects.
				controls = new ArrayList<Control>();
				System.out.println("[ShellObjectIterator]: Composite: " +composite+ ", number of children controls: " +compositeControls.length);
				//Iterate through the Control-objects, adding them to the ArrayList, and recursively iterate through any Composite-instances.
				for(int i=0; i < compositeControls.length; i++) {
					if( compositeControls[i] instanceof Composite ){
						//Recursively iterate through Composite.
						iterateComposite( (Composite)compositeControls[i] );
					}else{
						//Add Control-object to ArrayList.
						getControls().add( compositeControls[i] );
						System.out.println("[ShellObjectIterator]: compositeControls["+i+"]: " +compositeControls[i]+ ", is not an instance of Composite");
					}
				}//end for.
			}
		}
	}//end ShellObjectIterator.
	
	
	
	
	/**
	 * Get this.Shell.
	 * @return Shell, or null.
	 */
	private Shell getShell(){
		return this.shell;
	}
	
	/**
	 * Method that returns the Shell-objects Control-children.
	 * @return ArrayList<Control> of the Shell-objects children Control-objects, or null if the Shell does not have any Control-objects.
	 */
	public ArrayList<Control> getControls(){
		return this.controls;
	}	
	
	/**
	 * Method that iterates through a SWT Shell-object and adds them to the ArrayList of Control-objects.
	 * Searching iteratively through a Composite-instance from the Shell-object.
	 * Calls itself recursively when it discovers a Composite-object.
	 */
	private void iterateComposite(Composite composite){
		System.out.println("\n[ShellObjectIterator]: iterateComposite: " +composite);
		//Add Composite to ArrayList.
		getControls().add( composite );
		//Iterate through the Composite Control-objects.
		Control[] compositeChildren = composite.getChildren();
		if( compositeChildren != null ){
			for(int i=0; i < compositeChildren.length; i++) {
				System.out.println("compositeChildren["+i+"]: "+compositeChildren[i] );				
				//The Control inside the Composite is an instance of Composite.						
				if( compositeChildren[i] instanceof Composite ){
					System.out.println("compositeChildren["+i+"]: "+compositeChildren[i] + " is an instance of Composite!");
					//Recursively iterate through the Composite-instance.
					iterateComposite( (Composite)compositeChildren[i] );
				}else{
					//Add Control to ArrayList.
					getControls().add( compositeChildren[i] );
				}
			}// end for.
		}
	}//end iterateComposite();
	
}//ShellObjectIterator.

/* Fungerende kode:

				//The Control inside the Composite is a Composite.						
				if( compositeChildren[i].getClass().getSimpleName().equals("Composite") ){
					System.out.println("compositeChildren["+i+"]: "+compositeChildren[i] + " is an instance of Composite!");
					//Recursively iterate through the Composite.
					iterateComposite( (Composite)compositeChildren[i] );
				}else if( compositeChildren[i].getClass().getSimpleName().equals("FilteredTree") ){
					System.out.println("compositeChildren["+i+"]: "+compositeChildren[i] + " is an instance of Composite: " +(compositeChildren[i] instanceof Composite) );
					//Recursively iterate through the Composite.
					iterateComposite( (Composite)compositeChildren[i] );
				}else{
					//Add Control to ArrayList.
					getControls().add( compositeChildren[i] );
				}
				
 */
 
