package com.eclipse.listener.helpClasses;

import java.util.ArrayList;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;

import com.eclipse.listener.views.MouseEventListenerView;

public class WorkspaceMethods {
	
	
	//Metode som sjekker hvilken type et element er: Javaprosjekt, src-mappe, package, javaklasse,	
	//Metode som sjekker om et element med navn String er en src-mappe	
	//Metode som sjekker om et element med navn String er en Java-klasse 	
	
	// getPackages(String project)	- Alle packages til gitt project.
	
	
	/**
	 * Method that iterates through the workspace and returns the projects inside.
	 * @return 
	 * <b>ArrayList[IProject]</b> with projects. Returns null if there are no projects in the workspace.	
	 */
	public static ArrayList<IProject> getProjectsInWorkspace(){
		ArrayList<IProject> projects = null;		
		// Get the root of the workspace
	    IWorkspace workspace = ResourcesPlugin.getWorkspace();
	    IWorkspaceRoot root = workspace.getRoot();
	    // Get all projects in the workspace
	    IProject[] rootProjects = root.getProjects();
	    if(rootProjects.length > 0){
	    	projects = new ArrayList<IProject>();
	    	for (int i = 0; i < rootProjects.length; i++) {		    	
		    	projects.add( rootProjects[i] ); //project.
			}
	    }		
		return projects;
	}//getProjectsInWorkspace.
	
	/**
	 * Metod that returns an IProject with the name given as a parameter.
	 * @param name
	 * @return IProject, or null if the project with the given name does not exist.
	 */
	public static IProject getProject(String name){
		ArrayList<IProject> projects = getProjectsInWorkspace();
		if(projects != null){
			for(IProject iProject : projects) {
				if(iProject.getName().equals(name)){
					return iProject;
				}
			}
		}		
		return null;
	}//getProject.
	
	/**
	 * Method that checks if a project with the given name in the workspace is a Java Project.
	 * @param name
	 * @return boolean
	 * @throws CoreException
	 */
	public static boolean isAJavaProject(String name) throws CoreException{
		IProject project = getProject(name);
		if( project != null ){
			if (project.isNatureEnabled("org.eclipse.jdt.core.javanature")) {
				return true;
			}
		}		
		return false;
	}//isAJavaProject.
	
	/**
	 * Method for returning the packages for the workspace.
	 * @return ArrayList of IPackageFragments
	 * @throws CoreException
	 */
	public static ArrayList<IPackageFragment> getPackagesInWorkspace() throws CoreException{
		ArrayList<IPackageFragment> workspacePackages = new ArrayList<IPackageFragment>();
		ArrayList<IProject> projects = getProjectsInWorkspace();	//projects
		//Iterate through projects in the workspace.
		for(IProject iProject : projects) {
			if(isAJavaProject(iProject.getName())){					//only check for packages inside java-projects.
				IJavaProject javaProject = JavaCore.create(iProject);
				IPackageFragment[] packages = javaProject.getPackageFragments();
				//Iterate through packagefragments for a java-project.
				for(int i = 0; i < packages.length; i++) {
					if (packages[i].getKind() == IPackageFragmentRoot.K_SOURCE) {
						workspacePackages.add(packages[i]);						
					}
				}				  
			}
		}
		return workspacePackages;
	}//getPackagesInWorkspace.
	
	
	public static ArrayList<IPackageFragment> getPackagesInProject(String name) throws CoreException{
		ArrayList<IPackageFragment> projectPackages = new ArrayList<IPackageFragment>();
		ArrayList<IPackageFragment> workspacePackages = getPackagesInWorkspace();	//packages in workspace.
		if( workspacePackages != null ){
			//Iterate through packages in the workspace.
			for (IPackageFragment iPackageFragment : workspacePackages) {
				if( iPackageFragment.getJavaProject().getElementName().equals(name) ){			//package -> src -> project		
					projectPackages.add(iPackageFragment);
				}
			}
		}
		return projectPackages;
	}//getPackagesInProject.
	
	/**
	 * Method that checks if the workspace has a package with the name given by a parameter.
	 * @param <b>String</b>
	 * @return boolean
	 * @throws CoreException
	 */
	public static boolean isAPackage(String name) throws CoreException{
		IPackageFragment packageFragment = getPackage(name);
		//The package needs to be in the workspace, and have a parent folder named "src".
		if( packageFragment != null  && packageFragment.getParent() != null && packageFragment.getParent().getElementName().equals("src") ){
			return true;
		}
		return false;
	}//isAPackage.
	
	/**
	 * Method for returning a IPackageFragment (package) which has the name given by a parameter.
	 * @param name
	 * @return IPackageFragment, or null if it does not exist a package with the name given by a parameter.
	 * @throws CoreException
	 */
	public static IPackageFragment getPackage(String name) throws CoreException{
		ArrayList<IPackageFragment> packages = getPackagesInWorkspace();
		if( packages != null && packages.size() > 0 ){
			//Iterating through the packages.
			for(IPackageFragment iPackageFragment : packages) {
				//Returning the packagefragment related to the parameter name.
				if( iPackageFragment.getElementName().equals(name) ){
					return iPackageFragment;
				}
			}
		}
		return null;
	}//getPackage.
	
	/**
	 * Method that returns the java source file compilation unit that matches the given parameter.
	 * @param name - Ex: "File.java".
	 * @return ICompilationUnit, or null if there is no java source file with the given name.
	 * @throws CoreException
	 */
	public static ICompilationUnit getJavaSourceFile(String name) throws CoreException{
		//Packages for java-projects inside the workspace.
		ArrayList<IPackageFragment> packages = getPackagesInWorkspace();
		if(packages != null){
			for(int i = 0; i < packages.size(); i++) {
				//CompilationUnitDetails.
				for (ICompilationUnit unit : packages.get(i).getCompilationUnits()) {
					if( unit.getElementName().equals(name) ){
						return unit;
					}					
				}
			}
		}		
		return null;
	}//end getJavaSourceFile.
	
	/**
	 * Method that returns the package (parent) to the java source file.
	 * @param name - Ex: "File.java".
	 * @return IJavaElement, or null if there is no java source file that matches the given parameter.
	 * @throws CoreException
	 */
	public static IJavaElement getPackageToJavaSourceFile(String name) throws CoreException{
		ICompilationUnit javaFile = getJavaSourceFile(name);
		if( javaFile != null && javaFile.getElementName().equals(name) ){
			return javaFile.getParent();
		}
		return null;
	}// end getPackageToJavaSourceFile.
	
	/**
	 * Method that returns the java-project to the java source file.
	 * @param name
	 * @return IJavaElement, or null if there is no java source file that matches the given parameter.
	 * @throws CoreException
	 */
	public static IJavaElement getProjectToJavaSourceFile(String name) throws CoreException{
		ICompilationUnit javaFile = getJavaSourceFile(name);
		if( javaFile != null && javaFile.getElementName().equals(name) ){
			return javaFile.getJavaProject();
		}
		return null;
	}// end getProjectToJavaSourceFile.
	
	/**
	 * Method that checks if there is a java-file with the given name as a parameter.
	 * @param name
	 * @return boolean - true if the file is a java file or false if it is not a java file.
	 * @throws CoreException
	 */
	public static boolean isAJavaFile(String name) throws CoreException{
		if( getJavaSourceFile(name) != null){
			return true;
		}
		return false;
	}
	
	public static void getJavaSourceFileSettings(String name) throws CoreException{
		ICompilationUnit unit = getJavaSourceFile(name);
		if(unit != null){
			IType[] allTypes = unit.getAllTypes();
		    for(IType type : allTypes) {
		    	MouseEventListenerView.eventTextField.append("\nType: " + type.toString() );
		    	
		    	IMethod[] methods = type.getMethods();
		        for (IMethod method : methods) {
		          MouseEventListenerView.eventTextField.append("\nMethod name: " + method.getElementName());
		          MouseEventListenerView.eventTextField.append("\nSignature: " + method.getSignature());
		          MouseEventListenerView.eventTextField.append("\nReturn Type: " + method.getReturnType());
		        }
		    }
		}
		
	}
	
	/**
	 * Method for returning the files inside of a package with the given name.
	 * @param name
	 * @return ArrayList of IResource-files. Ex: "File.java", "Textfile.txt".
	 * @throws CoreException
	 */
	public static ArrayList<IResource> getFilesInPackage(String name) throws CoreException{
		ArrayList<IResource> packageFiles = new ArrayList<IResource>();
		IPackageFragment packageFragment = getPackage(name);		
		//Check if package exists in the workspace.
		if( packageFragment != null ){
			//Get the resource file to the package.
			IResource packageResource = packageFragment.getCorrespondingResource();			
			if(packageResource != null && packageResource.getType() == IResource.FOLDER){
				IFolder folder = (IFolder)packageResource;
				IResource[] temp = folder.members();
				for(int i = 0; i < temp.length; i++) {
					packageFiles.add(temp[i]);
				}
			}
		}		
		return packageFiles;
	}// getFilesInPackage.
	
	/**
	 * Method for getting a random Java Project from the workspace.
	 * @return IProject that is a Java Project, or null if there is no Java Project in the workspace.
	 */
	public static IProject getRandomJavaProjectFromWorkspace(){
		ArrayList<IProject> getProjectsInWorkspace = getProjectsInWorkspace();
		if( getProjectsInWorkspace != null ){
			for(IProject iProject : getProjectsInWorkspace) {
				try {
					if( isAJavaProject(iProject.getName()) ){
						return iProject;
					}
				} catch (CoreException e) {
					e.printStackTrace();
				}
			}
		}		
		return null;
	}//end getRandomJavaProjectFromWorkspace.
	
	/**
	 * Method for getting the name of a random Java Project from the workspace.
	 * @return String of a Java Project, or an empty string if there is no Java Project in the workspace.
	 */
	public static String getRandomJavaProjectNameFromWorkspace(){
		ArrayList<IProject> getProjectsInWorkspace = getProjectsInWorkspace();
		if( getProjectsInWorkspace != null ){
			for(IProject iProject : getProjectsInWorkspace) {
				try {
					if( isAJavaProject(iProject.getName()) ){
						return iProject.getName();
					}
				} catch (CoreException e) {
					e.printStackTrace();
				}
			}
		}		
		return "";
	}//end getRandomJavaProjectNameFromWorkspace.
	
	public static IFile getRandomJavaSourceFileFromWorkspace() throws CoreException{
		for( IPackageFragment packageItem : getPackagesInWorkspace() ) {
			System.out.println("Package: " + packageItem.getElementName());
			for (IJavaElement element : packageItem.getChildren()) {
				System.out.println("JavaElement: " + element.getElementName());
				if( element != null && isAJavaFile(element.getElementName()) ){
					return (IFile)element.getResource();
				}
			}			
		}		
		return null;
	}//end getRandomJavaSourceFileFromWorkspace.
	
	
	//skrive ut innhold i workspace.
	public static void printWorkspace() throws CoreException{
		MouseEventListenerView.eventTextField.append("\n\n"+Methods.getCurrentTime()+", PrintWorkspace:");
		ArrayList<IProject> workspaceProjects = WorkspaceMethods.getProjectsInWorkspace();


		//Alternativ1.
		//Projects.
		for (int i = 0; i < workspaceProjects.size(); i++) {
			MouseEventListenerView.eventTextField.append("\n\nproject["+i+"]: " + workspaceProjects.get(i).getName() );
			//Packages.
			ArrayList<IPackageFragment> pckes = getPackagesInProject( workspaceProjects.get(i).getName() );
			if( pckes != null){
				for (int j = 0; j < pckes.size(); j++) {
					MouseEventListenerView.eventTextField.append("\npackage["+j+"]: " + pckes.get(j).getElementName() );
					//Files.
					ArrayList<IResource> pckFiles = getFilesInPackage( pckes.get(j).getElementName() );
					if( pckFiles != null ){
						for(int k = 0; k < pckFiles.size(); k++) {
							MouseEventListenerView.eventTextField.append("\nfile["+k+"]: " + pckFiles.get(k).getName() );
						}
					}
				}
			}
			
		}		
		
		//Alternativ 2.
		//Iterate project
		for (int i = 0; i < workspaceProjects.size(); i++) {
			MouseEventListenerView.eventTextField.append("\nproject["+i+"]: " +workspaceProjects.get(i).getName() );
		
			//Iterate packages.
			ArrayList<IPackageFragment> projectPackages = getPackagesInProject(workspaceProjects.get(i).getName());
			for (int j = 0; j < projectPackages.size(); j++) {
				MouseEventListenerView.eventTextField.append("\n\tpackage["+j+"]: " +projectPackages.get(j).getElementName() );
			
				//Iterate Classes.
				for(int k = 0; k < projectPackages.size(); k++) {
					//CompilationUnitDetails.
					for (ICompilationUnit unit : projectPackages.get(k).getCompilationUnits()) {
						MouseEventListenerView.eventTextField.append("\n\t\tclass["+k+"]: " +unit.getElementName());
											
					}
				}
			}
		}
		
		//Projects in workspace.
		for(int i = 0; i < workspaceProjects.size(); i++) {
			MouseEventListenerView.eventTextField.append("\n\nproject["+i+"]: " +workspaceProjects.get(i).getName()+", resources: " +workspaceProjects.get(i).members().length );
			
			//Files/folders inside project.
			IResource[] projectChild = workspaceProjects.get(i).members();
			for (int j = 0; j < projectChild.length; j++) {
				MouseEventListenerView.eventTextField.append("\nprojectChild["+j+"]: " +projectChild[j].getName() +",   parent: "+projectChild[j].getParent().getName() );
				if ( projectChild[j].getType() == IResource.FOLDER ) {
					IFolder folder = (IFolder)projectChild[j];
					
					//src-folder inside project.
					if( folder.getName().toLowerCase().equals("src") ){
							
						//folders inside src-folder.
						IResource[] projectChildChild = folder.members();
						for (int k = 0; k < projectChildChild.length; k++) {
							MouseEventListenerView.eventTextField.append("\nprojectChildChild["+k+"]: " + projectChildChild[k].getName() +",   parent: "+projectChildChild[k].getParent().getName() );
							
							if( projectChildChild[k].getType() == IResource.FOLDER ){
								IFolder srcContentFolder = (IFolder)projectChildChild[k];
								IResource[] projectChildChildChild = srcContentFolder.members();
								for (int l = 0; l < projectChildChildChild.length; l++) {
									MouseEventListenerView.eventTextField.append("\nprojectChildChildChild["+l+"]: " + projectChildChildChild[l].getName() +",   parent: "+projectChildChildChild[l].getParent().getName() );

								}
							}
							
							
						}
						
					}
				}
			}
		}
		
		getFilesInPackage("yoloForever");
	}//printWorkspace.
	
}
