package com.eclipse.listener.scenario.task;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Event;

import com.eclipse.listener.scenario.task.autocomplete.AutoExecution;

/**
 * A subclass of SwtTask.java that detects when a user clicks on a dialog button. 
 * <br>ex: "Back", "Next", "Finish", "Cancel", "OK", "Browse".
 *@param String - the name of a dialog-button. Ex: "OK".
 */
public class DialogButtonClick extends SwtTask {
	/**
	 * Constructor with the name of a dialog button as parameter.
	 * @param dialogButton String - the name of a dialog-button. Ex: "OK".
	 */
	public DialogButtonClick(String dialogButton) {
		super(dialogButton, SWT.Selection);
	}
	
	/**
	 * Constructor with the name of a dialog button as parameter, and a description as a parameter.
	 * @param dialogButton String - the name of a dialog-button. Ex: "OK".
	 * @param description String
	 */
	public DialogButtonClick(String dialogButton, String description, AutoExecution autoExecuteTask) {
		super(dialogButton, description, SWT.Selection, autoExecuteTask);
	}
	
	/**
	 * Method containing the logic for verifying that the occurred Event finishes this Task.
	 */
	@Override
	protected boolean isTaskEvent(Event event) {
		System.out.println("["+getClass().getSimpleName()+"]: Selection-Event: " + event );
		//Button.
		if( event.widget instanceof Button ){
			Button button = (Button)event.widget;
			String buttonText = "";
			if( button.getText() != null ){
				buttonText = button.getText();			// Ex: "Cancel", "OK", "&Finish".
			}
			//Check if the text of the clicked dialog button equals the title of this Task.
			if( removeMnemonics(buttonText).contains(getTitle().toLowerCase()) ){
				System.out.println("["+getClass().getSimpleName()+"]: Task completed! Dialog-Button clicked: " + getTitle());
				//Finish the task: isFinished=true, isActivated=false, removeListeners().
				return true;
			}					
		}//Button.	
		return false;
	}//end isTaskEvent.

}//end DialogButtonClick.