package com.eclipse.listener.scenario.task;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.widgets.Event;

import com.eclipse.listener.helpClasses.Methods;
import com.eclipse.listener.helpClasses.WorkspaceMethods;
import com.eclipse.listener.scenario.task.autocomplete.AutoExecution;

/**
 * A subclass of SwtTask.java that detects when a user uses the keyboard to type a text string inside an editor.
 * <br><br><b>ex:</b> "Sysout" inside an editor for a java file.
 */
public class KeyboardTextType extends SwtTask {
	/**
	 * Constructor with keyboardText as a parameter.
	 * @param keyboardText String
	 */
	public KeyboardTextType(String keyboardText) {
		super(keyboardText, SWT.KeyUp);
	}

	/**
	 * Constructor with keyboardText as a parameter, and a description as a parameter.
	 * @param keyboardText String
	 * @param description String
	 */
	public KeyboardTextType(String keyboardText, String description, AutoExecution autoExecuteTask) {
		super(keyboardText, description, SWT.KeyUp, autoExecuteTask);
	}

	/**
	 * Method containing the logic for verifying that the occurred Event finishes this Task.
	 */
	@Override
	protected boolean isTaskEvent(Event event) {
		//Check if currently active window is a java file and that it contains "sysout".
		if( event.widget instanceof StyledText ){
			StyledText styledText = (StyledText)event.widget;
			if( styledText != null && styledText.getText() != null ){
				try {
					//Get the caret line-number and line-text.
					int currentLineNumber = styledText.getLineAtOffset(styledText.getCaretOffset()) + 1;	//+1 because it uses 0-indexing, Java Editor uses 1-indexing!
					String currentLineText = styledText.getLine(currentLineNumber - 1);						//-1 because it uses 0-indexing, Java Editor uses 1-indexing!			
					System.out.println("\ncurrentLineNumber: " + currentLineNumber);
					System.out.println("currentLineText:>" + currentLineText +"<");
					
					if( WorkspaceMethods.isAJavaFile(Methods.getCurrentlySelectedViewName()) && currentLineText.toLowerCase().contains("sysout") ){
						System.out.println("["+getClass().getSimpleName()+"]: Task completed! Typed 'sysout' in a Java File: " + Methods.getCurrentlySelectedViewName());
						//Finish the task: isFinished=true, isActivated=false, removeListeners().
						return true;
					}
				} catch (Exception e) {
					System.out.println("["+getClass().getSimpleName()+"]: isAJavaFile exception: " + e);
				}					
			}
		}//end StyledText.	
		return false;
	}//end isTaskEvent.

}//end KeyboardTextType.