package com.eclipse.listener.scenario.task;

import org.eclipse.core.commands.HandlerEvent;
import org.eclipse.jface.action.CoolBarManager;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.menus.IWidget;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Decorations;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.commands.ICommandService;
import org.eclipse.ui.handlers.HandlerUtil;
import org.eclipse.ui.handlers.IHandlerService;

import com.eclipse.listener.helpClasses.Methods;
import com.eclipse.listener.scenario.task.autocomplete.AutoExecution;

/**
 * A subclass of SwtTask.java that detects when a user clicks on one of the buttons on the menu bar: 
 * <br>"File", "Edit", "Source", "Refactor", "Navigate", "Search", "Project", "Run", "Window", "Help".
 *@param String - the name of a button in the menu bar. Ex: "Window".
 */
public class MenuBarClick extends SwtTask {

	/**
	 * Constructor with name of menu bar button as parameter.
	 * @param menuBarButton String - the name of a button in the menu bar. Ex: "Window".
	 */
	public MenuBarClick(String menuBarButton) {
		super(menuBarButton, SWT.Show);
	}
	
	/**
	 * Constructor with name of menu bar button as parameter, and a description as a parameter.
	 * @param menuBarButton String - the name of a button in the menu bar. Ex: "Window".
	 * @param description String
	 */
	public MenuBarClick(String menuBarButton, String description, AutoExecution autoExecuteTask) {
		super(menuBarButton, description, SWT.Show, autoExecuteTask);
	}	
	
	/**
	 * Method containing the logic for verifying that the occurred Event finishes this Task.
	 */
	@Override
	protected boolean isTaskEvent(Event event) {
/*		//TESTING - START.
		Point location = null;
		Menu menu1 = Methods.getDisplay().getActiveShell().getMenuBar();
		if(menu1 != null){
			System.out.println("MenuBar != null: " + menu1+ ", count: " + menu1.getItemCount()+ ", getData: " + menu1.getData());
			if( menu1.getData() != null && menu1.getData() instanceof MenuManager ){
				MenuManager menu1Manager = (MenuManager)menu1.getData();
				System.out.println( "Menu1Manager: " + menu1Manager+ ", items:"+menu1Manager.getItems().length );
				if( menu1Manager.getItems() != null ){
					for (IContributionItem element : menu1Manager.getItems()) {
						System.out.println("IContributionItem: id:" +element.getId() +"element: "+element );
					}
				}
			}			
			
			MenuItem[] menuItems = menu1.getItems();
			if( menuItems != null ){
				for(MenuItem menuItem : menuItems) {
					System.out.println("MenuItem: " + menuItem+", text: " +menuItem.getText()+ ", getData: " + menuItem.getData()+", getSelection: "+menuItem.getSelection());
				}
			}
			
			Decorations menubarDecorations = menu1.getParent();
			location = menubarDecorations.getLocation();
			if(location != null){
				Methods.getDisplay().setCursorLocation(location);
//				MessageDialog.openInformation(null, "Info Point Location", "Point Location : " + location);
			}
			System.out.println("menubarDecorations: " + menubarDecorations+ ", location: " + menubarDecorations.getLocation()+ ", getData: "+ menubarDecorations.getData());
			Control[]  menuBarDecorationsControls = menubarDecorations.getChildren();
			if( menuBarDecorationsControls != null ){
				for (Control control : menuBarDecorationsControls) {
					System.out.println("Control: " + control+ ", location:" + control.getLocation()+ ", getData: " + control.getData() + ", bounds: "+control.getBounds() + ", toDisplay: "+control.toDisplay(control.getLocation()));
					location = control.getLocation();
					if(location != null){
						Methods.getDisplay().setCursorLocation(location);
//						MessageDialog.openInformation(null, "Info Point Location", "Point Location : " + location);
					}
					
					if( control instanceof Composite ){
						Composite comp = (Composite)control;
						if( comp.getChildren() != null ){
							Control[] compChildren = comp.getChildren();
							for (Control control2 : compChildren) {
								System.out.println("compChildren: " + control2+ ", location: "+control2.getLocation());
								location = control2.getLocation();
								if(location != null){
									Methods.getDisplay().setCursorLocation(location);
//									MessageDialog.openInformation(null, "Info Point Location", "Point Location : " + location);
								}
							}
						}
						
					}
				}
			}

			MenuItem mainMenuItem = menu1.getItems()[0];
			System.out.println("\nTESTING:");
			System.out.println("Event info: " + event+ ", x: "+event.x+ ", y: "+event.y);
			System.out.println("decorations: " + menu1.getParent()+ ", bounds: "+menu1.getParent().getBounds()+", location: "+menu1.getParent().getLocation()+", toDisplay: "+menu1.getParent().toDisplay(menu1.getParent().getLocation()));
			System.out.println("mainMenu: " + menu1+ ", menuitemsCount: " + menu1.getItemCount());
			System.out.println("mainMenuItem0: "+mainMenuItem+ ", text: " + mainMenuItem.getText() +", selection:" + mainMenuItem.getSelection());
			
		}//menu1
		//TESTING - STOP.
*/
		
		System.out.println("["+getClass().getSimpleName()+"]: Show-Event: " + event );				
		//Menu.
		if( event.widget instanceof Menu ){
			Menu menu = (Menu)event.widget;
			String menuParentItemText = ""; // Ex: &File, &Edit, &Source, Refac&tor, &Navigate, Se&arch, &Project, &Run, &Window, &Help.
			if( menu.getParentItem() != null ){
				menuParentItemText = menu.getParentItem().getText();
				System.out.println("["+getClass().getSimpleName()+"]: MenuParentItemText: " + menuParentItemText +", getData: " + menu.getParentItem().getData());
			}					
			//Check if the text of the clicked menu bar button equals the title of this Task.
			if( removeMnemonics(menuParentItemText).contains(getTitle().toLowerCase()) ){
				System.out.println("["+getClass().getSimpleName()+"]: Task completed! Button clicked on the menu bar: " + getTitle());
				//Finish the task: isFinished=true, isActivated=false, removeListeners().
				return true;
			}										
		}//Menu.
		return false;
	}//end isTaskEvent.
	
}//end MenuBarClick.