package com.eclipse.listener.scenario.task;

import java.util.ArrayList;
import java.util.Arrays;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;

import com.eclipse.listener.scenario.task.autocomplete.AutoExecution;

/**
 * A subclass of SwtTask.java that detects when a user clicks on one of the MenuItems of the menu bar.
 * <br><b>SWT.Show:</b> If the MenuItem has a Menu with MenuItems, like "Show View", it uses a listener with event type SWT.Show.
 * <br><b>SWT.Selection:</b> If the MenuItem does not have a Menu with MenuItems, like "Preferences", it uses a listener with event type SWT.Selection.
 */
public class MenuBarItemClick extends SwtTask {
	private static ArrayList<String> menuItemsWithMenu = new ArrayList<String>(
	Arrays.asList("&Editor", "&Open Perspective", "Show &View", "Navi&gation"));	
	
	private static ArrayList<String> menuItemsWithoutMenu = new ArrayList<String>(
	Arrays.asList("&New Window", "Show &Toolbar", "Hide &Toolbar", "Customi&ze Perspective...", 
			"Save Perspective &As...", "&Reset Perspective...", "&Close Perspective", "Close A&ll Perspectives", "&Preferences",
			"&New	Alt+Shift+N", "&Close	Ctrl+W", "Clea&n...", "Console	Alt+Shift+Q", "Debug", "&Properties"));
	
	//File:		&New	Alt+Shift+N, Open File&..., |, &Close	Ctrl+W, C&lose All	Ctrl+Shift+W, |, &Save	Ctrl+S, Save &As..., Sav&e All	Ctrl+Shift+S, Rever&t, |, Mo&ve..., Rena&me...	F2, Re&fresh	F5, Con&vert Line Delimiters To, |, &Print...	Ctrl+P, |, Switch &Workspace, Restart, |, &Import..., Exp&ort..., |, P&roperties	Alt+Enter, |, E&xit
	//Edit:		&Undo	Ctrl+Z, &Redo	Ctrl+Y, |, Cu&t	Ctrl+X, &Copy	Ctrl+C, Cop&y Qualified Name, &Paste	Ctrl+V, |, &Delete	Delete, Select &All	Ctrl+A, E&xpand Selection To, |, &Find/Replace...	Ctrl+F, Find &Next, Find Pre&vious, &Incremental Find Next, Incre&mental Find Previous, |, Add Bookmar&k..., Add Ta&sk..., |, Sma&rt Insert Mode, |, Show T&ooltip Description, |, Co&ntent Assist, &Word Completion, &Quick Fix	Ctrl+1, |, Set Encodin&g...
	//Source:	Togg&le Comment, Add &Block Comment, Remove Bloc&k Comment, Generate Element Comm&ent	Alt+Shift+J, |, &Shift Right, S&hift Left, Correct &Indentation, &Format, Format Eleme&nt, |, A&dd Import, Or&ganize Imports	Ctrl+Shift+O, S&ort Members..., Clean &Up..., |, O&verride/Implement Methods..., Gene&rate Getters and Setters..., Generate Delegate &Methods..., Generate to&String()..., Generate &hashCode() and equals()..., Gener&ate Constructor using Fields..., Generate &Constructors from Superclass..., |, Surround &With	Alt+Shift+Z, |, E&xternalize Strings..., Find Broken Ex&ternalized Strings
	//Refactor:	Re&name...	Alt+Shift+R, &Move...	Alt+Shift+V, |, &Change Method Signature...	Alt+Shift+C, E&xtract Method...	Alt+Shift+M, Extract &Local Variable...	Alt+Shift+L, Extr&act Constant..., &Inline...	Alt+Shift+I, |, Convert Local Varia&ble to Field..., C&onvert Anonymous Class to Nested..., Mo&ve Type to New File..., |, Extrac&t Superclass..., &Extract Interface..., Use Supertype W&here Possible..., Push &Down..., Pull &Up..., |, Extract Class..., Introduce &Parameter Object..., |, Introduce Indirec&tion..., Introduce &Factory..., Introduce &Parameter..., Encap&sulate Field..., |, Generali&ze Declared Type..., Infer &Generic Type Arguments..., |, Migrate &JAR File..., C&reate Script..., Appl&y Script..., &History...
	//Navigate:	Go &Into, &Go To, |, &Open	F3, Ope&n Type Hierarchy	F4, Open Call H&ierarchy	Ctrl+Alt+H, Open Hyperlin&k, O&pen Implementation, Open &Super Implementation, Open Attached &Javadoc	Shift+F2, Open from Clipboar&d, |, Switch Source/Design Editors, Open &Type...	Ctrl+Shift+T, Open Type in Hierarch&y...	Ctrl+Shift+H, Open Reso&urce...	Ctrl+Shift+R, |, Show in &Breadcrumb, Sho&w In	Alt+Shift+W, Quick Out&line, Quick Type &Hierarchy, |, Ne&xt	Ctrl+., Pre&vious	Ctrl+,, |, Last Edit Lo&cation	Ctrl+Q, &Go to Line..., |, &Back	Alt+Left, &Forward	Alt+Right
	//Search:	Se&arch...	Ctrl+H, &File..., &Java..., |, Te&xt, |, R&eferences, Dec&larations, &Implementors, &Read Access, &Write Access, |, Occurre&nces in File	Ctrl+Shift+U, |, Referring &Tests...
	//Project:	Op&en Project, Clo&se Project, |, Build &All	Ctrl+B, &Build Project, Build &Working Set, Clea&n..., Build Auto&matically, |, &Generate Javadoc..., |, &Properties
	//Run: 		&Run	Ctrl+F11, &Debug	F11, |, Run His&tory, Run A&s, Ru&n Configurations..., |, Debug &History, Debu&g As, De&bug Configurations..., |, Toggle Brea&kpoint	Ctrl+Shift+B, Toggle &Line Breakpoint, Toggle &Method Breakpoint, Toggle &Watchpoint, S&kip All Breakpoints, Remo&ve All Breakpoints, Add Java E&xception Breakpoint..., Add Class Load Break&point..., |, All &References..., All Ins&tances...	Ctrl+Shift+N, Instance Co&unt..., Wa&tch, Insp&ect	Ctrl+Shift+I, Displa&y	Ctrl+Shift+D, E&xecute	Ctrl+U, &Force Return	Alt+Shift+F, |, &External Tools
	//Window: 	&New Window, &Editor, Show &Toolbar, |, &Open Perspective, Show &View, |, Customi&ze Perspective..., Save Perspective &As..., &Reset Perspective..., &Close Perspective, Close A&ll Perspectives, |, Navi&gation, |, &Preferences
	//Help:		&Welcome, |, &Help Contents, S&earch, &Dynamic Help, |, &Key Assist...	Ctrl+Shift+L, &Tips and Tricks..., &Cheat Sheets..., |, Check for &Updates, Install New &Software..., &Installation Details, Eclipse &Marketplace..., |, &About Eclipse Platform
	
	/**
	 * Constructor with the name of the MenuItem as a parameter.
	 * @param menuItem String
	 */
	public MenuBarItemClick(String menuItem) {
		super(menuItem, findSwtEventType(menuItem));
	}	

	/**
	 * Constructor with the name of the MenuItem as a parameter, and description as a parameter.
	 * @param menuItem String
	 * @param description String
	 */
	public MenuBarItemClick(String menuItem, String description, AutoExecution autoExecuteTask) {
		super(menuItem, description, findSwtEventType(menuItem), autoExecuteTask);
	}
	
	/**
	 * Method for finding and returning the appropriate SWT-eventType for the MenuBarItem-Listener.
	 * <br><b>SWT.Show:</b> If the MenuItem has a Menu with MenuItems, like "Show View", we need to use a listener with event type SWT.Show.
	 * <br><b>SWT.Selection:</b> If the MenuItem does not have a Menu with MenuItems, like "Preferences", we use a listener with event type SWT.Selection.
	 * @param title String - The title of the MenuItem. Ex: "&Preferences", for a MenuItem without a Menu.
	 */
	private static int findSwtEventType(String title) {
		//If the MenuItem has a Menu with MenuItems, then add (SWT.Show)
		for(String menuItem : menuItemsWithMenu) {
			if( removeMnemonics(menuItem).contains( title.toLowerCase() ) ){
				return SWT.Show;
			}
		}
		//If the MenuItem does NOT have a Menu with MenuItems, then add (SWT.Selection)
		for(String menuItem : menuItemsWithoutMenu) {
			if( removeMnemonics(menuItem).contains( title.toLowerCase() ) ){
				return SWT.Selection;
			}
		}		
		return 0;		
	}//end findEventType.
	
	/**
	 * Method containing the logic for verifying that the occurred Event finishes this Task.
	 */
	@Override
	protected boolean isTaskEvent(Event event) {
		String eventTypeStr = "Event";
		if(event.type == 13){
			eventTypeStr = "Selection-Event";
		}else if(event.type == 22){
			eventTypeStr = "Show-Event";
		}				
		System.out.println("["+getClass().getSimpleName()+"]: " +eventTypeStr+ ": " + event );
		
		//MenuItem - A MenuItem without a Menu. Ex: "&Preferences".
		if( event.widget instanceof MenuItem ){
			MenuItem menuItem = (MenuItem)event.widget;
			String menuItemText = "";
			if( menuItem!=null ){
				menuItemText = menuItem.getText();		//Ex: &Preferences.
			}
			System.out.println("menuItemText: "+menuItemText);
			//Check if the text of the clicked MenuItem equals the title of this Task.
			if( removeMnemonics(menuItemText).contains(getTitle().toLowerCase()) ){
				System.out.println("["+getClass().getSimpleName()+"]: Task completed! MenuItem clicked on the menu bar: " + getTitle());
				//Finish the task: isFinished=true, isActivated=false, removeListeners().
				return true;
			}
		}//MenuItem.
		
		//Menu - A Menu that is also a MenuItem which has a Menu. Ex: "&File".
		else if( event.widget instanceof Menu ){
			Menu menu = (Menu)event.widget;
			String menuParentItemText = ""; // MenuParentItemText:   &File
			if( menu!=null && menu.getParentItem()!=null ){
				menuParentItemText = menu.getParentItem().getText();
			}
			System.out.println("menuParentItemText: "+menuParentItemText);
			//Check if the text of the clicked MenuItem with MenuItems, equals the title of this Task.
			if( removeMnemonics(menuParentItemText).contains(getTitle().toLowerCase()) ){
				System.out.println("["+getClass().getSimpleName()+"]: Task completed! MenuItem clicked on the menu bar: " + getTitle());
				//Finish the task: isFinished=true, isActivated=false, removeListeners().
				return true;
			}
		}//Menu.
		return false;
	}//end isTaskEvent.

}//end MenuBarItemClick.