package com.eclipse.listener.scenario.task;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.ToolItem;

import com.eclipse.listener.scenario.task.autocomplete.AutoExecution;

/**
 * A subclass of SwtTask.java that detects when a user clicks on one of the buttons on the tool bar: 
 * <br>"New", "Save", "Save All", "Print", "Debug", "Run", "New Java Class", are some examples.
 *@param String - the name of a button in the tool bar. Ex: "New".
 */
public class ToolBarClick extends SwtTask {

	/**
	 * Constructor with the name of a tool bar button as parameter.
	 * @param toolBarButton String
	 */
	public ToolBarClick(String toolBarButton) {
		//Calls the super constructor method with SWT.Selection as eventType.
		super(toolBarButton, SWT.Selection);
	}

	/**
	 * Constructor with the name of a tool bar button as parameter, and a description as a parameter.
	 * @param toolBarButton String
	 * @param description String
	 */
	public ToolBarClick(String toolBarButton, String description, AutoExecution autoExecuteTask) {
		//Calls the super constructor method with SWT.Selection as eventType.
		super(toolBarButton, description, SWT.Selection, autoExecuteTask);
	}
	
	/**
	 * Method containing the logic for verifying that the occurred Event finishes this Task.
	 */
	@Override
	protected boolean isTaskEvent(Event event) {
		System.out.println("["+getClass().getSimpleName()+"]: Selection-Event: " + event );
		//ToolItem - A button on the ToolBar is a ToolItem.
		if( event.widget instanceof ToolItem ){
			ToolItem toolItem = (ToolItem)event.widget;
			String toolItemTooltipText = "", toolItemText = "";
			//See if the ToolItem has a tooltip-text.
			if( toolItem.getToolTipText() != null ){
				toolItemTooltipText = toolItem.getToolTipText();	//Ex: "New", "Save (Ctrl+S)", "Print (Ctrl+P)", "Run", "Open Perspective".
			}
			//See if the ToolItem has a text in itself.
			if( toolItem.getText() != null ){
				toolItemText = toolItem.getText();
			}			
			//Check if the text/tooltip-text of the clicked tool bar button equals the title of this Task.
			boolean tooltipTextEqualsTaskTitle = removeMnemonics(toolItemTooltipText).contains(getTitle().toLowerCase());
			boolean textEqualsTaskTitle = removeMnemonics(toolItemText).contains(getTitle().toLowerCase());
			if( tooltipTextEqualsTaskTitle || textEqualsTaskTitle ){
				System.out.println("["+getClass().getSimpleName()+"]: Task completed! Button clicked on the tool bar: " + getTitle());
				//Finish the task: isFinished=true, isActivated=false, removeListeners().
				return true;
			}			
		}//ToolItem.
		return false; 				
	}//isTaskEvent.

}//end ToolBarClick.