package com.eclipse.listener.scenario.task;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

import com.eclipse.listener.helpClasses.Methods;
import com.eclipse.listener.scenario.task.autocomplete.AutoExecution;

/**
 * A subclass of Task.java that detects when a user clicks on a TreeItem. 
 * <br>ex: A Java Project, Package, Java Class in the Package Explorer. "Java Project" in the "New"-wizard.
 *@param String - the name of a TreeItem. Ex: "Java Project".
 */
public class TreeItemClick extends SwtTask {
	/**
	 * Constructor with the name of a TreeItem as a parameter.
	 * @param treeItem String - the name of a TreeItem. Ex: "Java Project".
	 */
	public TreeItemClick(String treeItem) {
		super(treeItem, SWT.Selection);
	}
	
	/**
	 * Constructor with the name of a TreeItem as a parameter, and a description as a parameter.
	 * @param treeItem String - the name of a TreeItem. Ex: "Java Project".
	 * @param description String
	 */
	public TreeItemClick(String treeItem, String description, AutoExecution autoExecuteTask) {
		super(treeItem, description, SWT.Selection, autoExecuteTask);
	}
	
	/**
	 * Method containing the logic for verifying that the occurred Event finishes this Task.
	 */
	@Override
	protected boolean isTaskEvent(Event event) {
		System.out.println("["+getClass().getSimpleName()+"]: Selection-Event: " + event );
		//Tree
		if( event.widget instanceof Tree ){
			Tree tree = (Tree)event.widget;
			TreeItem selectedTreeItem = Methods.getSelectedTreeItem(tree); 			// Get the single selected TreeItem from the Tree.			
			String treeItemText = "";
			if( selectedTreeItem != null && selectedTreeItem.getText() != null ){
				treeItemText = selectedTreeItem.getText();							//Ex: "Java Project".
			}
			//Check if the text of the clicked TreeItem equals the title of this Task.
			if( removeMnemonics(treeItemText).contains(getTitle().toLowerCase()) ){
				System.out.println("["+getClass().getSimpleName()+"]: Task completed! TreeItem clicked: " + getTitle());
				//Finish the task: isFinished=true, isActivated=false, removeListeners().
				return true;
			}
		}//Tree.
		return false;
	}//end isTaskEvent.

}//end TreeItemClick.