package com.eclipse.listener.scenario.task;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

import com.eclipse.listener.helpClasses.Methods;
import com.eclipse.listener.helpClasses.WorkspaceMethods;
import com.eclipse.listener.scenario.task.autocomplete.AutoExecution;

/**
 * A subclass of SwtTask.java that detects when a user clicks on a TreeItem and if it is a Java Project. 
 * <br>ex: "MyFirstJavaProject" in Package Explorer. 
 */
public class TreeItemClickIsAJavaProject extends SwtTask {
	/**
	 * Constructor with the name of a TreeItem as a parameter.
	 * @param treeItem String
	 */
	public TreeItemClickIsAJavaProject(String treeItem) {
		super(treeItem, SWT.Selection);
	}

	/**
	 * Constructor with the name of a TreeItem as a parameter, and a description as a parameter.
	 * @param treeItem String
	 * @param description String
	 */
	public TreeItemClickIsAJavaProject(String treeItem, String description, AutoExecution autoExecuteTask) {
		super(treeItem, description, SWT.Selection, autoExecuteTask);
	}	

	/**
	 * Method containing the logic for verifying that the occurred Event finishes this Task.
	 */
	@Override
	protected boolean isTaskEvent(Event event) {
		System.out.println("["+getClass().getSimpleName()+"]: Selection-Event: " + event );
		//Tree
		if( event.widget instanceof Tree ){
			Tree tree = (Tree)event.widget;
			TreeItem selectedTreeItem = Methods.getSelectedTreeItem(tree); 			// Get the single selected TreeItem from the Tree.			
			String treeItemText = "";
			if( selectedTreeItem != null && selectedTreeItem.getText() != null ){
				treeItemText = selectedTreeItem.getText();							//Ex: "Java Project".
			}
			//Check if the selected TreeItem is a java project(and exists in the workspace).
			try {
				if( WorkspaceMethods.isAJavaProject(treeItemText) ){
					System.out.println("["+getClass().getSimpleName()+"]: Task completed! TreeItem is a java project: " + getTitle());
					//Finish the task: isFinished=true, isActivated=false, removeListeners().
					return true;
				}
			} catch (CoreException e) {
				System.out.println("["+getClass().getSimpleName()+"]: CoreException: " + e);
				e.printStackTrace();
			}
		}//Tree.
		return false;
	}//end isTaskEvent.

}//end TreeItemClickIsAJavaProject.