package com.eclipse.listener.scenario.task.autocomplete;

import java.util.ArrayList;
import java.util.Arrays;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MenuItem;

import com.eclipse.listener.helpClasses.Methods;

public class ContextMenuItemClickExecution extends AutoExecution {
	//Variables.
	private Listener armListener;
	private int clickCounter = 0;
	private boolean pressDownArrow = false;
	
	/**
	 * Constructor
	 * @param title
	 */
	public ContextMenuItemClickExecution(String title) {
		super(title);
		//Create a SWT.Arm Listener to listen after menuitems getting focused by arrow-down-clicks.
		createArmListener();
	}
	
	/**
	 * Method for creating the SWT Arm Listener.
	 */
	private void createArmListener(){
		armListener = new Listener() {			
			@Override
			public void handleEvent(Event event) {
				System.out.println("\n[ContextMenuItemClickExecution]: Arm Event: " + event);
				if( event.widget instanceof MenuItem ){
					MenuItem menuitem = (MenuItem)event.widget;
					String menuitemText = Methods.removeMnemonics(menuitem.getText());

					Methods.threadSleep(150);
					if( menuitemText.contains(getTitle().toLowerCase()) ){
						Methods.getDisplay().removeFilter(SWT.Arm, armListener);
						pressDownArrow = false;
						System.out.println(menuitemText + " == " + getTitle().toLowerCase());
						executeEnterClick();
					}
					else{
						clickCounter++;
						if(pressDownArrow && clickCounter < 30){
							executeDownArrowClick();
						}else{
							Methods.getDisplay().removeFilter(SWT.Arm, armListener);
						}							
					}						
				}
			}//MenuItem.			
		};
	}//createArmListener.
	
	/**
	 * Help method for executing a keyboard arrow down click.
	 */
	private void executeDownArrowClick(){
		Methods.getDisplay().asyncExec(new Runnable() {
			public void run() {
				Methods.simulateKeyboardKeyPress(SWT.ARROW_DOWN);
				System.out.println("\nexecuteDownArrowClick!, clickcounter: " + clickCounter);
			}
		});		
	}//end executeDownArrowClick.
	
	/**
	 * Help method for executing a keyboard enter click.
	 */
	private void executeEnterClick(){
		Methods.getDisplay().asyncExec(new Runnable() {
			public void run() {
				Methods.threadSleep(500);
				Methods.simulateKeyboardKeyPress(SWT.CR);
				System.out.println("\nexecuteEnterClick!, clickcounter: " + clickCounter);
			}
		});		
	}//end executeEnterClick.
	
	@Override
	public void execute() {
		clickCounter = 0;
		EclipseCommandExecution scenario4Task2 = new EclipseCommandExecution("Context Menu Java Project");
		scenario4Task2.execute();
		
//		//Executing the tasks 1 and 2 for scenario 4.
//		Methods.getDisplay().asyncExec(new Runnable() {
//			public void run() {
//				EclipseCommandExecution scenario4Task2 = new EclipseCommandExecution("Context Menu Java Project");
//				scenario4Task2.execute();			
//			}
//		});		
		
		Methods.getDisplay().asyncExec(new Runnable() {
			public void run() {
				Methods.threadSleep(300);
				//Add armListener to the Display-object.
				Methods.getDisplay().addFilter(SWT.Arm, armListener);
				
				//Press keyboard down button. This is to activate the arm listener.	
				pressDownArrow = true;
				executeDownArrowClick();			
			}
		});			
				
	}//end execute.

}//ContextMenuItemClickExecution.