package com.eclipse.listener.scenario.task.autocomplete;

import org.eclipse.jdt.internal.ui.packageview.PackageExplorerPart;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

import com.eclipse.listener.helpClasses.Methods;
import com.eclipse.listener.helpClasses.WorkspaceMethods;

/**
 * Class for setting focus to the Package Explorer, checking if the workspace contains at least one project which type is Java Project
 * using the down-keyboard-button to select a project until a Java Project is selected.  
 */
@SuppressWarnings("restriction")
public class JavaProjectInPackageExplorerExecution extends AutoExecution {
	
	public JavaProjectInPackageExplorerExecution() {
		super("JavaProjectInPackageExplorerExecution");
	}

	@Override
	public void execute() {
		//Open Package Explorer View.
		openPackageExplorerView();		
		
		//Check if the workspace has at least one project of type Java Project.		
		final String nameOfRandomJavaProject = WorkspaceMethods.getRandomJavaProjectNameFromWorkspace();
		if( !nameOfRandomJavaProject.isEmpty() ){
			System.out.println("Workspace is not empty, random javaProject: " + nameOfRandomJavaProject);
			//Get Package Explorer part of the perspective, and select the java project that matches "nameOfRandomJavaProject".
			PackageExplorerPart part = PackageExplorerPart.getFromActivePerspective();	//part.selectAndReveal(WorkspaceMethods.getRandomJavaProjectFromWorkspace());
			if( part != null ){
				Tree packageExplorerTree = part.getTreeViewer().getTree();
				if( packageExplorerTree != null && packageExplorerTree.getItemCount() > 0 ){				
					TreeItem[] items = packageExplorerTree.getItems();
					for (TreeItem treeItem : items) {
						System.out.println("TreeItem: " + treeItem.getText()+ ", bounds: " +treeItem.getBounds() );						
						if( treeItem.getText().equals(nameOfRandomJavaProject) ){
							Methods.getDisplay().setCursorLocation(Methods.getCoordinatesOfTreeItem(treeItem));
							Methods.simulateMouseClick();
							break;
						}
					}				
				}
			}					
		}else{
			//The workspace does not contain any projects of type Java Project.
			MessageDialog.openError(null, "Error!", "The workspace does not contain any projects of type Java Project. Please create a Java Project first!");
		}
	}//end execute.
	
	/**
	 * Help method for open the Package Explorer view.
	 */
	private void openPackageExplorerView() {
		System.out.println("Fr openPackageExplorerView - Active Part Title: " + Methods.getActivePart().getTitle());
		//Open Package Explorer View.
		EclipseCommandExecution openPackageExplorer = new EclipseCommandExecution("Package Explorer");
		openPackageExplorer.execute();
		System.out.println("Etter openPackageExplorerView - Active Part Title: " + Methods.getActivePart().getTitle());		
	}

}//end JavaProjectInPackageExplorerExecution.