package com.eclipse.listener.scenario.task.autocomplete;

import java.util.ArrayList;
import java.util.Arrays;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MenuItem;

import com.eclipse.listener.helpClasses.Methods;

public class MenuBarItemClickExecution extends AutoExecution {
	//Variables.
	private ArrayList<String> fileMenuItems = new ArrayList<String>();
	private ArrayList<String> editMenuItems = new ArrayList<String>();
	private ArrayList<String> sourceMenuItems = new ArrayList<String>();
	private ArrayList<String> refactorMenuItems = new ArrayList<String>();
	private ArrayList<String> navigateMenuItems = new ArrayList<String>();
	private ArrayList<String> searchMenuItems = new ArrayList<String>();
	private ArrayList<String> projectMenuItems = new ArrayList<String>();
	private ArrayList<String> runMenuItems = new ArrayList<String>();
	private ArrayList<String> windowMenuItems = new ArrayList<String>();
	private ArrayList<String> helpMenuItems = new ArrayList<String>();
	
	private Listener armListener;
	private int clickCounter = 0;
	private boolean pressDownArrow = true;
	
	/**
	 * Constructor
	 * @param title
	 */
	public MenuBarItemClickExecution(String title) {
		super(title);
		initMenuItemsIntoArrayLists();
		//Create a SWT.Arm Listener to listen after menuitems getting focused by arrow-down-clicks.
		createArmListener();
	}
	
	/**
	 * Method for filling ArrayLists with title of the MenuItems.
	 */
	private void initMenuItemsIntoArrayLists() {
		projectMenuItems.addAll(Arrays.asList("&Properties"));
		
		windowMenuItems.addAll(Arrays.asList("&New Window", "&Editor", "Show &Toolbar", "&Open Perspective", "Show &View", 
				"Customi&ze Perspective...", "Save Perspective &As...", "&Reset Perspective...", "&Close Perspective",
				"Close A&ll Perspectives", "Navi&gation", "&Preferences", "Debug", "Console	Alt+Shift+Q"));
	}//end initMenuItemsIntoArrayLists.
		
	/**
	 * Help method for checking if a given ArrayList contains a given String. 
	 * @param arrayList
	 * @param string
	 * @return true if the ArrayList contains a given String, false otherwise.
	 */
	private boolean arrayListContains(ArrayList<String>arrayList, String string){
		for(String str : arrayList) {
			if( Methods.removeMnemonics(str).contains(string.toLowerCase()) ){
				return true;
			}
		}		
		return false;
	}//end arrayListContains.
	
	/**
	 * Method for creating the SWT Arm Listener.
	 */
	private void createArmListener(){
		armListener = new Listener() {			
			@Override
			public void handleEvent(Event event) {
				System.out.println("\n[MenuBarItemClickExecution]: Arm Event: " + event);
				if( event.widget instanceof MenuItem ){
					MenuItem menuitem = (MenuItem)event.widget;
					String menuitemText = Methods.removeMnemonics(menuitem.getText());
					ArrayList<String> menuBarButtons = new ArrayList<String>();
					menuBarButtons.addAll(Arrays.asList("file","edit","source","refactor","navigate","search","project","run","window","help"));
					//Only press down arrow button if the armed element is a menuitem and not a menubar-element.
					if( !menuBarButtons.contains(menuitemText) ){
						Methods.threadSleep(150);
						if( getTitle().toLowerCase().equals("console") && menuitemText.contains("show view") ){
							executeEnterClick();
						}
						else if( getTitle().toLowerCase().equals("debug") && menuitemText.contains("open perspective") ){
							executeEnterClick();
						}
						else if( menuitemText.contains(getTitle().toLowerCase()) ){
							Methods.getDisplay().removeFilter(SWT.Arm, armListener);
							pressDownArrow = false;
							System.out.println(menuitemText + " == " + getTitle().toLowerCase());
							executeEnterClick();
						}
						else{
							clickCounter++;
							if(pressDownArrow && clickCounter < 20){
								executeDownArrowClick();
							}else{
								Methods.getDisplay().removeFilter(SWT.Arm, armListener);
							}							
						}						
					}
				}//MenuItem.
			}
		};
	}//createArmListener.
	
	/**
	 * Help method for creating a MenuBarClickExecution with the given String, and executing it.
	 * @param menuBarButton
	 */
	private void executeMenuBarClick(final String menuBarButton){
		Methods.getDisplay().asyncExec(new Runnable() {
			public void run() {
				MenuBarClickExecution menuBarClick = new MenuBarClickExecution(menuBarButton);
				menuBarClick.execute();
				System.out.println("\nexecuteMenuBarClick!");
			}
		});		
	}//end executeMenuBarClick.
	
	/**
	 * Help method for executing a keyboard arrow down click.
	 */
	private void executeDownArrowClick(){
		Methods.getDisplay().syncExec(new Runnable() {
			public void run() {
				Methods.simulateKeyboardKeyPress(SWT.ARROW_DOWN);
				System.out.println("\nexecuteDownArrowClick!, clickcounter: " + clickCounter);
			}
		});		
	}//end executeDownArrowClick.
	
	/**
	 * Help method for executing a keyboard enter click.
	 */
	private void executeEnterClick(){
		Methods.getDisplay().syncExec(new Runnable() {
			public void run() {
				Methods.threadSleep(500);
				Methods.simulateKeyboardKeyPress(SWT.CR);
				System.out.println("\nexecuteEnterClick!, clickcounter: " + clickCounter);
			}
		});		
	}//end executeEnterClick.
	
	@Override
	public void execute() {
		clickCounter = 0;
		pressDownArrow = true;
		
		//Find and execute the correct MenuBarClick, based on getTitle() MenuItem.
		if( arrayListContains(projectMenuItems, getTitle()) ){			//MenuItem is a Project-MenuItem.
			executeMenuBarClick("Project");								//Execute Project mouse click.
		}
		else if( arrayListContains(windowMenuItems, getTitle()) ){		//MenuItem is a Window-MenuItem.
			executeMenuBarClick("Window");								//Execute Window mouse click.						
		}
		
		//Add armListener to the Display-object.
		Methods.getDisplay().addFilter(SWT.Arm, armListener);
		
		//Press keyboard down button. This is to activate the arm listener.		
		executeDownArrowClick();
		
	}//end execute.	
}//end MenuBarItemClickExecution.