package com.eclipse.listener.scenario.task.autocomplete;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

import com.eclipse.listener.helpClasses.Methods;

public class TreeItemClickExecution extends AutoExecution {
	//Variables.
	private Listener selectionListener;
	private int clickCounter = 0;
	private boolean pressPrefixButton = false;
	private char prefixTreeItem;	//Ex: if title is "Java Build Path", the prefix will be 'j'. Then the j-button on the keyboard will be simulated.
	
	/**
	 * Constructor
	 * @param title
	 */
	public TreeItemClickExecution(String title) {
		super(title);
		prefixTreeItem = title.toLowerCase().charAt(0);
		//Create a SWT.Selection Listener to listen after a treeitem getting focused by pressing the first letter of the treeitem.
		createSelectionListener();
	}
	
	/**
	 * Method for creating the SWT Selection Listener.
	 */
	private void createSelectionListener(){
		selectionListener = new Listener() {			
			@Override
			public void handleEvent(Event event) {
				System.out.println("\n[TreeItemClickExecution]: Selection Event: " + event);
				if( event.widget instanceof Tree ){
					Tree tree = (Tree)event.widget;
					TreeItem selectedTreeItem = Methods.getSelectedTreeItem(tree);
					String selectedTreeItemText ="";
					if( selectedTreeItem != null ){
						selectedTreeItemText = selectedTreeItem.getText().toLowerCase();
					}
					System.out.println("Selected Tree Item: " + selectedTreeItemText);
	
					Methods.threadSleep(10);
					if( selectedTreeItemText.contains(getTitle().toLowerCase()) ){
						Methods.getDisplay().removeFilter(SWT.Selection, selectionListener);
						pressPrefixButton = false;
						System.out.println(selectedTreeItemText + " == " + getTitle().toLowerCase());
					}
					else{
						clickCounter++;
						if(pressPrefixButton && clickCounter < 15){
							executeKeyboardPrefixButtonClick();
						}else{
							Methods.getDisplay().removeFilter(SWT.Selection, selectionListener);
						}							
					}						
				}//Tree.
				else{
					clickCounter++;
					if(pressPrefixButton && clickCounter < 15){
						executeKeyboardPrefixButtonClick();
					}else{
						Methods.getDisplay().removeFilter(SWT.Selection, selectionListener);
					}							
				}
			}//handleEvent.			
		};
	}//createArmListener.
	
	/**
	 * Help method for executing a keyboard arrow down click.
	 */
	private void executeDownArrowClick(){
		Methods.getDisplay().asyncExec(new Runnable() {
			public void run() {
				Methods.simulateKeyboardKeyPress(SWT.ARROW_DOWN);
				System.out.println("executeDownArrowClick!, clickcounter: " + clickCounter);
			}
		});		
	}//end executeDownArrowClick.
	
	/**
	 * Help method for executing a keyboard click on the button with the prefix of the title. Ex: "Java Build Path" gives the prefix 'j'.
	 */
	private void executeKeyboardPrefixButtonClick(){
		Methods.getDisplay().asyncExec(new Runnable() {
			public void run() {
				Methods.threadSleep(500);
				Methods.simulateKeyboardKeyPress( (int)prefixTreeItem );
				System.out.println("\nExecuteKeyboardClick with prefix: " +prefixTreeItem+ ", clickcounter: " + clickCounter);
			}
		});		
	}//end executeKeyboardPrefixButtonClick.
	
	@Override
	public void execute() {
		clickCounter = 0;		
		ContextMenuItemClickExecution scenario4 = new ContextMenuItemClickExecution("Properties");	
		scenario4.execute();
//		//Executing the tasks 1,2,3 for scenario 4.
//		Methods.getDisplay().asyncExec(new Runnable() {
//			public void run() {
//				ContextMenuItemClickExecution scenario4 = new ContextMenuItemClickExecution("Properties");	
//				scenario4.execute();
//			}
//		});		
		
		Methods.getDisplay().asyncExec(new Runnable() {
			public void run() {
				Methods.threadSleep(100);
				//Add selectionListener to the Display-object.
				Methods.getDisplay().addFilter(SWT.Selection, selectionListener);
				
				//Press keyboard down button. This is to activate the selection listener.	
				pressPrefixButton = true;
				executeDownArrowClick();
				executeKeyboardPrefixButtonClick();
			}
		});			
				
	}//end execute.

}//end TreeItemClickExecution.