package com.eclipse.listener.views;

import java.awt.Robot;
import java.util.ArrayList;

import javax.swing.JOptionPane;
import javax.swing.UIManager;

import org.eclipse.core.commands.Command;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IExecutionListener;
import org.eclipse.core.commands.NotEnabledException;
import org.eclipse.core.commands.NotHandledException;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.MenuEvent;
import org.eclipse.swt.events.MenuListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.ScrollBar;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IPerspectiveDescriptor;
import org.eclipse.ui.IPerspectiveListener;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.commands.ICommandService;
import org.eclipse.ui.handlers.IHandlerService;
import org.eclipse.ui.part.*;
import org.eclipse.ui.wizards.IWizardRegistry;

import com.eclipse.listener.helpClasses.Methods;	//Inneholder statiske hjelpemetoder.
import com.eclipse.listener.helpClasses.ScenarioHandler;

public class MouseEventListenerView extends ViewPart {
	// Variables.
	private Label eventTextFieldLabel, eventTextFieldStatusLabel, searchInEventTextFieldLabel; 
	public static Text eventTextField, searchInEventTextField;
	private Button startButton, stopButton, clearButton, mouseCursorCheckButton, searchButton, showAllEventsButton;
	private static boolean showEventsInEventTextField = false;
	private static int eventCounter = 0;
	private static ArrayList<String> eventTextFieldContent = new ArrayList<String>();	
	
	public static ArrayList<String> achievements = new ArrayList<String>();	// Inneholder oppndde achievements.
	public int middleMouseButtonClicks = 0;
	public int currentScenario = 0; //Blir brukt til  vise hvilket scenario som brukeren har aktivert.
	
	//Listeners.
	public Listener mouseMoveListener, mouseUpListener, selectionListener, menuDetectListener, showListener, expandListener, closeListener, keyUpListener;
	public IPerspectiveListener perspectiveListener;
	public ISelectionListener iSelectionListener;
	public IPartListener iPartListener;
	
	public IExecutionListener iexelistener;
	
	public ICommandService commandService;	
	
	
	//Robot Testing
	public Robot bot = null;

	//Wizard testing.
	public static IWizardRegistry wizards = null;
	
	//createPartControl
	@Override
	public void createPartControl(Composite parent) {
		
		// Set the look and feel to the users system look and feel.
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			System.out.println("UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName()), success");
		} catch (Exception e) {
			System.out.println("\n\n"+Methods.getCurrentTime()+", Exception(UIManager look and feel): " + e.toString());
			e.printStackTrace();
		}
			
		// View Layout - the layout for the Composite.
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 3;
		parent.setLayout(gridLayout);
				
		// Event Text field label - the label above the event text field.
		eventTextFieldLabel = new Label(parent, SWT.NONE);
		eventTextFieldLabel.setText("Mouse Events:");
		eventTextFieldLabel.setToolTipText("Label for the event text field");
		GridData gridData = new GridData(SWT.BEGINNING, SWT.CENTER, false, false);
		gridData.horizontalSpan = 3;
		eventTextFieldLabel.setLayoutData(gridData);
		
		// Event text field - text field for showing events that occur.
		eventTextField = new Text(parent, SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL);
		gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
		gridData.horizontalSpan = 3;
		eventTextField.setEditable(false);		
		eventTextField.setToolTipText("Event text field for mouse events");
		eventTextField.setLayoutData(gridData);	
		
		// Event text field status label - displays whether or not to print new events to the event text field.
		eventTextFieldStatusLabel = new Label(parent, SWT.NONE);
		eventTextFieldStatusLabel.setText("Listening after events: [" +showEventsInEventTextField+ "]");
		eventTextFieldStatusLabel.setToolTipText("Status for listening after events");
		gridData = new GridData(SWT.BEGINNING, SWT.CENTER, true, false);
		gridData.horizontalSpan = 3;
		eventTextFieldStatusLabel.setLayoutData(gridData);
			
		// Start Button - makes it possible to print the mouse events into the event text field.
		startButton = new Button(parent, SWT.PUSH);
		startButton.setText("Start");
		startButton.setToolTipText("Start printing the mouse events");
		gridData = new GridData(SWT.BEGINNING, SWT.CENTER, false, false);
		startButton.setLayoutData(gridData);
		startButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event){
				showEventsInEventTextField = true;
				updateEventTextFieldStatusLabel();
				// Adding listeners.
				addListeners();	
			}
		});
		
		// Stop Button - stops making it possible to print the mouse events into the event text field.
		stopButton = new Button(parent, SWT.PUSH);
		stopButton.setText("Stop");
		stopButton.setToolTipText("Stop listing the mouse events");
		gridData = new GridData(SWT.BEGINNING, SWT.CENTER, false, false);
		stopButton.setLayoutData(gridData);
		stopButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event){
				showEventsInEventTextField = false;
				updateEventTextFieldStatusLabel();
				
			}
		});
		
		// Clear Button - clear the text in the event text field.
		clearButton = new Button(parent, SWT.PUSH);
		clearButton.setText("Clear");
		clearButton.setToolTipText("Clear the text field");
		gridData = new GridData(SWT.BEGINNING, SWT.CENTER, false, false);
		clearButton.setLayoutData(gridData);
		clearButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event){
				clearTextField(eventTextField);
			}
		});
		
		// Checkbox Button - show the mouse cursor location events.
		mouseCursorCheckButton = new Button(parent, SWT.CHECK);	
		mouseCursorCheckButton.setText("Show mouse location");
		mouseCursorCheckButton.setToolTipText("Show or hide mouse location events");
		gridData = new GridData(SWT.BEGINNING, SWT.CENTER, false, false);
		gridData.horizontalSpan = 3;
		mouseCursorCheckButton.setSelection(false);
		mouseCursorCheckButton.setLayoutData(gridData);
		
		// Search event text field label - the label above the search event text field.
		searchInEventTextFieldLabel = new Label(parent, SWT.NONE);
		searchInEventTextFieldLabel.setText("Search field for the event text field:");
		eventTextFieldLabel.setToolTipText("Label for the search event text field");
		gridData = new GridData(SWT.BEGINNING, SWT.CENTER, false, false);
		gridData.horizontalSpan = 3;
		searchInEventTextFieldLabel.setLayoutData(gridData);
		
		// searchInTextField Text - used to search after content in the event text field.		
		searchInEventTextField = new Text(parent, SWT.BORDER);
		gridData = new GridData(SWT.FILL, SWT.CENTER, false, false);
		gridData.horizontalSpan = 2;
		searchInEventTextField.setEditable(true);		
		searchInEventTextField.setToolTipText("Text field for searching after events in the event text field");
		searchInEventTextField.setLayoutData(gridData);
		
		// Search Button - use the keywords to search in the event text field.
		searchButton = new Button(parent, SWT.PUSH);
		searchButton.setText("Search");
		searchButton.setToolTipText("Search the event text field");
		gridData = new GridData(SWT.BEGINNING, SWT.CENTER, false, false);
		gridData.horizontalSpan = 1;
		searchButton.setLayoutData(gridData);
		searchButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event){
				// Do not want to search with an empty search field.
				if(!searchInEventTextField.getText().isEmpty()){
					clearTextField(eventTextField);
					int numberOfResults = 0;
					String searchString = searchInEventTextField.getText().toLowerCase();
					eventTextField.append("Search results:");
					for(int i=0; i<eventTextFieldContent.size(); i++) {
						if(eventTextFieldContent.get(i).toLowerCase().contains(searchString)){
							numberOfResults++;
							eventTextField.append(eventTextFieldContent.get(i));
						}
					}
					
					// Append a message if there is no results matching the search query.
					if(numberOfResults <= 0){
						eventTextField.append("\nThere is no results for the given query: " + searchString);
						eventTextField.append("\nTry another query, or show all events by clicking on " + showAllEventsButton.getText());
					}
				}

			}
		});
		
		// Show All Button - show all events that has been printed into the text field.
		showAllEventsButton = new Button(parent, SWT.PUSH);
		showAllEventsButton.setText("Show all events");
		showAllEventsButton.setToolTipText("Show the events that has been printed into the event text field");
		gridData = new GridData(SWT.BEGINNING, SWT.CENTER, false, false);
		gridData.horizontalSpan = 1;
		showAllEventsButton.setLayoutData(gridData);
		showAllEventsButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event){
				clearTextField(eventTextField);
				printEventTextFieldContent();
			}
		});
		
		
		
		
		/**
		 * LISTENERS.............................................................................................................................................................
		 */	
		iSelectionListener = new ISelectionListener() {			
			@Override
			public void selectionChanged(IWorkbenchPart sourcepart, ISelection selection) {
				String contentDescription = sourcepart.getTitle() + " (" + selection.getClass().getName() + ")";			
				//Selection of elements. Ex: Elements inside the "Package Explorer".
				if (selection instanceof IStructuredSelection) {
					IStructuredSelection ss = (IStructuredSelection)selection;
					eventTextField.append("\n"+Methods.getCurrentTime()+", IStructuredSelection:");
					eventTextField.append("\ncontentDescription: "+contentDescription);
					Object[] ssArray = ss.toArray();
					for (int i = 0; i < ssArray.length; i++) {
						eventTextField.append("\nssArray: " + ssArray[i]);
					}
					
					eventTextField.append("\nSELECTED_ELEMENT: " + ss.getFirstElement());
				}
				//Selection of text. Ex: Marked text inside an editor.
				else if (selection instanceof ITextSelection) {
					ITextSelection ts  = (ITextSelection) selection;
					eventTextField.append("\n\n"+Methods.getCurrentTime()+", ITextSelectionGetText:");
					eventTextField.append("\ncontentDescription: "+contentDescription);
					eventTextField.append("tsGetText: "+ ts.getText()+"\n");
				}
				else{
					eventTextField.append("\n\n"+Methods.getCurrentTime()+", OTHER-selectionChanged:\n"+sourcepart + "\n"+ selection); 
				}
			}
		};
		//getSite().getWorkbenchWindow().getSelectionService().addSelectionListener(iSelectionListener);//adding listener.
		
		//Legger til en lytter p workbench som registrerer at en part blir aktivert. Ex: "ExampleClass.java" blir pnet eller satt fokus til.
		iPartListener = new IPartListener() {			
			@Override
			public void partOpened(IWorkbenchPart part) {
			}			
			@Override
			public void partDeactivated(IWorkbenchPart part) {				
			}			
			@Override
			public void partClosed(IWorkbenchPart part) {				
			}			
			@Override
			public void partBroughtToTop(IWorkbenchPart part) {				
			}			
			@Override
			public void partActivated(IWorkbenchPart part) {
				//System.out.println("\n\npartActivated: " + part+ "\nTitle: " + part.getTitle());
				ScenarioHandler.scenarioVerify(null);
			}
		};		
		
		//Legger p en lytter p workbench som registrerer at et perspektiv blir aktivert. Ex: "Java", "Debug" blir aktivert.
		perspectiveListener = new IPerspectiveListener() {			
			@Override
			public void perspectiveChanged(IWorkbenchPage page, IPerspectiveDescriptor perspective, String changeId) {
			}
			
			//Kalles nr et perspektiv blir aktivert. Feks: nr man har trykket og kommet inn p Debug-perspective.
			@Override
			public void perspectiveActivated(IWorkbenchPage page, IPerspectiveDescriptor perspective) {
				eventTextField.append("\n\n"+Methods.getCurrentTime()+ " perspectiveActivated\nPage: " +page.getLabel()+ "\nPerspective: "+perspective.getLabel());
				System.out.println( "\n\n"+Methods.getCurrentTime()+ " perspectiveActivated\nPage: " +page.getLabel()+ "\nPerspective: "+perspective.getLabel() );
												
				//Sjekke om dette er frste gang dette perspektivet er blitt pnet.
				if( achievements.contains( perspective.getLabel() + "-perspective" ) ){		//Ex: "Java-perspective", "Debug-perspective"
					eventTextField.append("\n"+perspective.getLabel()+ "-Perspective achievement allerede oppndd!");
				}else{
					achievements.add( perspective.getLabel() + "-perspective" ); 	//legger til perspektivet som achievement i listen med oppndde achievement. 
					eventTextField.append("\nDu pner n perspektivet: " + perspective.getLabel() );
					
					//Popup-vindu som gir brukeren info om at det blir pnet et perspektiv.
					MessageDialog.openInformation(null, "Du pner n perspektivet: " + perspective.getLabel(), "GRATULERER!");
					
					//Oppdatere AchievementView til  vise en ny oppndd achievement.
//					ScenarioView.achievementTextField.append("\n\npne: " + perspective.getLabel() + " perspektivet" );
//					Methods.updateAchievementLabelScore();
				}
			}			
		};
		
		
		
		// Mouse Move Listener on the workbench window. Activates whenever the mouse moves.
		mouseMoveListener = new Listener() {			
			@Override
			public void handleEvent(Event event) {
				if(mouseCursorCheckButton.getSelection()){
					showMouseCursorLocation();
				}			
			}
		};		
		
		// Mouse Up Listener on the workbench window. Blir kalt i det yeblikket en museknapp som har blitt trykket ned slippes.
		mouseUpListener = new Listener() {			
			@Override
			public void handleEvent(Event event) {
				if( showEventsInEventTextField ){
					handleMouseUp(event);		// Hndtere event ved museklikk.
					eventTextField.append("\nCurrent Perspective: " + Methods.getCurrentPerspectiveName() + ", " + Methods.getCurrentTime());	// Skrive ut current perspective. ex: "Java", "Debug"
					eventTextField.append("\nCurrently Selected View: " + Methods.getCurrentlySelectedViewName() ); 					// Skrive ut currently selected view.
					eventTextField.append("\nCurrently Opened Dialog: " + Methods.getCurrentlyOpenedDialogWindowName() );				// Skrive ut currently opened dialog window name.
				}
/*				//Get selection of the selected(opened) view in Eclipse workbench. Ex:
				ISelection selection = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActivePartReference().getPage().getActivePart().getSite().getSelectionProvider().getSelection();
				AchievementView.achievementTextField.append(selection.getClass().getCanonicalName() + "\n" + selection.getClass().getName() + "\n" + selection.getClass().getTypeName()+ "\n" + selection.getClass().getTypeParameters());
*/ // **m finne ut hvordan man henter ut selection element og selection text.				
				
			}			
		};			
	    
		// Selection Listener on the workbench window. Lytter til om noe blir valgt. Feks: Bruke musepekeren til  trykke p et java-prosjekt, klikke p en "OK"-knapp.
		selectionListener = new Listener() {			
			@Override
			public void handleEvent(Event event) {
				if( showEventsInEventTextField ){
					handleSelection(event);
				}
			}
		}; 			
		
		// MenuDetect Listener on the workbench window.
		menuDetectListener = new Listener() {			
			@Override
			public void handleEvent(Event event) {
				if( showEventsInEventTextField ){
					appendText("MenuDetect Event(getDisplay().addFilter): \n" + event.toString());
				}
			}
		};
					
		// Show Listener on the workbench window.
		showListener = new Listener() {			
			@Override
			public void handleEvent(Event event) {
				if( showEventsInEventTextField ){
					appendText("Show Event: \n" + event.toString());
					
					//Sjekke om event.widget er instanceof Menu
					if(event.widget instanceof Menu){
						Menu menu = (Menu)event.widget;
						String menuParentItemText = ""; // MenuParentItemText:   &File
						try {							
							if( menu.getParentItem() != null ){	//unng nullpointer exception.
								menuParentItemText = menu.getParentItem().getText();
								eventTextField.append("\nMenuParentItemID:   " + menu.getParentItem().getID());	// MenuParentItemID:	0
								eventTextField.append("\nMenuParentItemText:   " + menuParentItemText);			// MenuParentItemText:   &File
							}
							if( menu.getParent() != null ){ 	//unng nullpointer exception.
								eventTextField.append("\nMenuParentDecorationText:   " +menu.getParent().getText());	// MenuParentDecorationText:   Java - Eclipse Platform
								eventTextField.append("\nMenuParentDecorationTooltipText:   " +menu.getParent().getToolTipText());
							}
							
							MenuItem[] menuItems = menu.getItems();
							ArrayList<String> menuItemsName = new ArrayList<String>();
							for (int i = 0; i < menuItems.length; i++) {
								menuItemsName.add( menuItems[i].getText() );
							}
							eventTextField.append("\nMenu items count: " +menuItemsName.size()+ "\nmenuItemsName:" + menuItemsName );
						} catch (Exception e) {
							System.out.println("\n\n"+Methods.getCurrentTime()+", Exception(Show Event Menu)"+e);
							e.printStackTrace();
						}//try-catch						
						
						//Sjekke om hendelsen hrer til et scenario.
						ScenarioHandler.scenarioVerify(menu);					
						
					}//menu
					
					//Sjekke om event.widget er instanceof Shell
					else if( event.widget instanceof Shell ){
						Shell shell = (Shell)event.widget;
						eventTextField.append("Show Event: " +shell+ ", shellText: " + shell.getText() );
						eventTextField.append("[MouseEventListenerView]: Show Event: " +shell+ ", Shell-text: " +shell.getText() );
						eventTextField.append("Currently opened dialog window: " +Methods.getCurrentlyOpenedDialogWindow().getText() );
						//"New"-dialog window.
					}
					
					//Sjekke om event.widget er instanceof Composite
					else if( event.widget instanceof Composite ){
						try {
							Composite composite = (Composite)event.widget;
							eventTextField.append("\nComposite shell.getText:   " +composite.getShell().getText() );
							Composite composite2 = composite.getParent();
							if(composite2 != null && composite2.getShell() != null){
								eventTextField.append("\nComposite.parent shell.getText:   " +composite2.getShell().getText() );
							}
						} catch (Exception e) {
							System.out.println(Methods.getCurrentTime()+ ", Exception(Show Event Composite): " + e.toString());
							e.printStackTrace();
						}
	
					}//composite
					
				}//showeventsintextfield.
			}//handle event.
		};		
		
		// Expand Listener on the workbench window.
		expandListener = new Listener() {			
			@Override
			public void handleEvent(Event event) {
				if( showEventsInEventTextField ){
					appendText("Expand Event: \n" + event.toString());
					/*TreeItem root = (TreeItem)event.item;
					TreeItem[] tree = root.getItems();
					
					appendText("item: " + root.toString() + "\nItems in root Count: " + root.getItemCount() + "\nRoot getItems: " + root.getItems().toString());
					appendText("Tree: " + tree.toString() + "\nTree Length: " + tree.length);
					appendText("TREE EVENT:");
					for(int i = 0; i < tree.length; i++) {
						appendText(tree[i].toString());
					} */
				}
			
			}// handle event.
		};
		
		// When the user clicks on the "X" to close a window, or presses the Escape-button on the keyboard.
		closeListener = new Listener() {			
			@Override
			public void handleEvent(Event event) {				
				if( showEventsInEventTextField ){
					// Hvis event.widget er av typen Shell.
					if( event.widget instanceof Shell ){
						Shell shell = (Shell)event.widget;	// shell/window
						String title = shell.getText();		// title of shell/window
						
						//Fr en slik close-event i det man kjrer com.eclipse.listener og det pnes en ny instans av eclipse med pluginen installert.
						if(title.equals("Eclipse Platform")){
							System.out.println("\n"+Methods.getCurrentTime()+", Close Event[Eclipse Platform]: " +event);
						}else{
							String stringAchievement = title + "-close by crossing out";
							System.out.println("Shell: " + shell.toString() + "\nTitle: " + title);
							
							//Sjekke om denne achievementen allerede er oppndd.
							if( achievements.contains(stringAchievement) ){
								eventTextField.append("\n"+stringAchievement+ " achievement allerede oppndd!");
							}else{
								achievements.add(stringAchievement); //legger til achievementen i achievements nr denne er oppndd.
								eventTextField.append("\n" +stringAchievement+ "!");
								
								//Popup-vindu som gir brukeren info om at shell/vinduet ble lukket ved  krysset det ut.
								String info = "CloseEvent: " + event;
								info += "\n\nLukke: "+title+ ", ved  krysse det ut!";
								JOptionPane.showMessageDialog(null, info, "GRATULERER!", JOptionPane.PLAIN_MESSAGE);
								
								//Oppdatere AchievementView til  vise en ny oppndd achievement.
//								ScenarioView.achievementTextField.append("\n\nLukke: "+title+ ", ved  krysse det ut!");
//								Methods.updateAchievementLabelScore();
							}
						}
	
					}//shell.
					else{
						appendText("Close Event: \n" + event.toString());
//						ScenarioView.achievementTextField.append("\n\nClose Event: " + event.toString());
//						Methods.updateAchievementLabelScore();
						System.out.println("Close Event: \n" + event.toString());
					}//else	
				}//showeventsintextfield
			}//handle event.
		};
				
		//Keybord keys.....................................................................................................................................
		
		//KeyUp listener.
		keyUpListener = new Listener() {			
			@Override
			public void handleEvent(Event event) {
				System.out.println("KeyUpEvent: char: " +event.character+", keycode:"+event.keyCode);
				if( showEventsInEventTextField ){					
//					Methods.keyUpEventHandler(event);	//Handling the key up event.					
				}//showeventsintextfield
			}//handle event
		};		
		
		
		
/*		//Skrive ut innhold i workspace.
		try {
			WorkspaceMethods.printWorkspace();
		} catch (CoreException e) {
			e.printStackTrace();
		}
*/

	}//end createPartControl.
	
	
	/**
	 * Method for handling a Mouse Up Event.
	 * @param event
	 */
	public void handleMouseUp(Event event) {
		
		//LEFT MOUSECLICK.
		if(event.button == 1){	
//			Methods.mouseUpEventLeftClick(event);			
		}//leftmouseclick.
		
		//WHEEL MOUSECLICK.
		else if(event.button == 2){	
			//Nr det er blitt klikket 5 ganger p denne knappen, skal brukeren oppn en skjult achievement med et tips om eclipse.
			middleMouseButtonClicks++;
			if(middleMouseButtonClicks == 5){
				achievements.add("middleMouseButtonClicks-5");
				appendText("MouseUpEvent(Wheel Click): " + event+ "\nmiddleMouseButtonClicks: " + middleMouseButtonClicks);
				
				//Popup-vindu som gir brukeren info om at det er klikket 5 ganger p den midtre museknappen.
				String tips = 	"Visste du at du ogs kan lukke en fane ved \nplassere musepekeren over fane-tittelen og\nklikke p den midtre museknappen!";
				String content = "Du har klikket p den midtre museknappen " +middleMouseButtonClicks+ " ganger.\n\nTips:\n" +tips;
				JOptionPane.showMessageDialog(null, content, "GRATULERER!", JOptionPane.PLAIN_MESSAGE);
				
				//Oppdatere AchievementView til  vise en ny oppndd achievement.
//				ScenarioView.achievementTextField.append("\n\n" +middleMouseButtonClicks+ " klikk p midtre museknapp");
//				Methods.updateAchievementLabelScore();				
			}else{
				appendText("MouseUpEvent(Wheel Click): " + event);
			}
			
		}
		
		//RIGHT MOUSECLICK.
		else if(event.button == 3){	
			appendText("MouseUpEvent(Right Click): " + event);
		}
		
		//BACK MOUSECLICK.
		else if(event.button == 4){	
			
// Tenkte  legge til testfunksjonlitet som pner/shows views, funksjoner, Shells, MenuItems osv.
// Feks: Klikke p back-mouse-button for  pne menyen "New Java Class".			
			
			//Open new-wizard.
			String wizardID = JOptionPane.showInputDialog("Skriv inn ID til nsket wizard du nsker  pne:");
			if(wizardID != null && !wizardID.isEmpty()){
				Methods.openNewWizardDialogWindow(wizardID);
			}
			
			try {
				//PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().showView("org.eclipse.ui.console.ConsoleView"); // Setter fokus til Console-view, hvis det ikke er lagt til et Console-view blir dette lagt til automatisk.
				
			} catch (Exception e) {
				e.printStackTrace();
				System.out.println(e);
			}			
			appendText("MouseUpEvent(Back Click): " + event);
		}
		
		//FORWARD MOUSECLICK.
		else if(event.button == 5){	
			/*try {				
				//org.eclipse.ui.dialogs
				//PreferencesUtil.createPropertyDialogOn(null, null, null, null, null).open(); // pne "Properties". ???? M teste ut. -Funker ikke med 5*null-parametre, gir nullpointer.
								
				PreferencesUtil.createPreferenceDialogOn(null, null, null, null).open(); // pne "Preferences". -Funker.
			} catch (Exception e) {
				e.printStackTrace();
				System.out.println(e);
			}
			*/
			appendText("MouseUpEvent(Forward Click): " + event);

//TODO
//TESTING			
			eventTextField.append("\nMain Menu: " + org.eclipse.ui.menus.MenuUtil.MAIN_MENU );
			
			eventTextField.append("\n\n");			
			
			String fileNewCommandID = org.eclipse.ui.IWorkbenchCommandConstants.FILE_NEW;
			eventTextField.append("\norg.eclipse.ui.IWorkbenchCommandConstants.FILE_NEW: " +fileNewCommandID);
			
			//ICommandService			
			eventTextField.append("\nCommandService: " + commandService);
			Command[] commands = commandService.getDefinedCommands();			
			eventTextField.append("\nCommand:");
			org.eclipse.core.commands.Command cmd = commandService.getCommand("org.eclipse.ui.newWizard");
			if(cmd != null){
				eventTextField.append("\nCommand cmd:\n " + cmd+"\n\n");
			}
			
			//Choose command from list.
			String[] selectionValues = 
				{ 
					"org.eclipse.ui.newWizard",									//open the "New" wizard selection window.
					"org.eclipse.ui.file.import",								//import wizard.
					"org.eclipse.ui.file.export", 								//export wizard.
					"org.eclipse.ui.perspectives.showPerspective",				//Show a particular perspective. Shows the "Open Perspective"-window.
					"org.eclipse.ui.views.showView",							//Shows a particular view. Shows the "Show View"-window.
					"org.eclipse.ui.edit.copy", 								//copy selection to clipboard.
					"org.eclipse.ui.edit.paste",								//paste from clipboard.
					"org.eclipse.ui.edit.cut",									//cut selection to clipboard.
					"org.eclipse.ui.edit.selectAll", 							//select all.
					"org.eclipse.ui.edit.delete",								//Delete the selection. Ex: file(s) or selected text/item(s).
					"org.eclipse.ui.edit.findReplace",							//open "find and replace"- window.					
					"org.eclipse.ui.edit.text.contentAssist.proposals",			//Content assistant.
					"org.eclipse.ui.file.print",								//File -> print.
					"org.eclipse.ui.file.close",								//File -> close.	
					"org.eclipse.ui.window.preferences",						//Window -> Preferences. Open the preferences dialog.
					"org.eclipse.ui.file.properties",							//File -> Properties. Display the properties of the selected item.
					"org.eclipse.ui.editors.lineNumberToggle",					//Toggle the line numbers on/off in the left side of an editor.
					"org.eclipse.jdt.ui.edit.text.java.toggleMarkOccurrences",	//Toggles mark occurrences in Java editors. Ex: If you mark a variable, this will mark the variable's occurences. 
					"org.eclipse.ui.edit.text.toggleShowWhitespaceCharacters",	//Shows whitespace characters in current text editor.
					"org.eclipse.ui.edit.text.goto.line",						//Opens the "go to line" window.
					"org.eclipse.jdt.ui.edit.text.java.rename.element",			//Rename the selected item. Ex: A java-project, java-package, java-file.
					"org.eclipse.ltk.ui.refactoring.commands.renameResource",	//Rename the selected resource and notify LTK participants. Ex: A txt-file.
					"org.eclipse.ui.edit.rename",								//Rename the selected item. Hotkey: F2. Works on all file types that can be renamed.
					"org.eclipse.ui.edit.undo",									//Undo the last operation.
					"org.eclipse.ui.edit.redo",									//Redo the last operation.
					"org.eclipse.jdt.ui.edit.text.java.create.getter.setter",	//Generate Getter and Setter methods for type's fields. Opens the "Generate getters and setters"-window.
					"org.eclipse.jdt.ui.edit.text.java.surround.with.try.catch",//Surround the selected text with a try/catch block.
					"org.eclipse.jdt.ui.edit.text.java.add.unimplemented.constructors",		//Evaluate and add constructors from superclass.
					"org.eclipse.jdt.ui.edit.text.java.generate.constructor.using.fields",	//Choose fields to initialize and constructor from superclass to call.
					"org.eclipse.ui.file.restartWorkbench",						//Restart the workbench. File -> Restart.
					"org.eclipse.ui.file.exit",									//Exit the eclipse-application. File -> Exit.
					"org.eclipse.ui.edit.move",									//Move the selected item. File -> Move.
					"org.eclipse.ui.file.refresh",								//Refresh the selected items. File -> Refresh. Hotkey: F5.
					"org.eclipse.jdt.ui.edit.text.java.toggle.comment",			//Toggle comment the selected line(s). Hotkey: Ctrl+7. 
					"org.eclipse.jdt.ui.edit.text.java.indent",					//Corrects the indentation of the selected lines.
					"org.eclipse.jdt.ui.edit.text.java.add.javadoc.comment",	//Add a Javadoc comment stub to the member element. Ex: Generates javadoc comment stub for a field/method where the cursor is placed.
					"org.eclipse.ui.edit.text.shiftRight",						//Shift a block of text to the right. Ex: Right-Tabbing selected text in an editor.
					"org.eclipse.ui.edit.text.shiftLeft",						//Shift a block of text to the left. Ex: Left-Tabbing selected text in an editor.
					"org.eclipse.ui.edit.addTask",								//Add a task on the selected line in the editor.
					"org.eclipse.ui.navigate.goToResource",						//Go to a particular resource in the active view. Ex: Select a task in the task-view, right-click and choose "go to".
					"org.eclipse.ui.ide.markCompleted",							//Mark the selected tasks as completed.
					"org.eclipse.ui.ide.deleteCompleted",						//Delete the tasks marked as completed.
					"org.eclipse.jdt.debug.ui.localJavaShortcut.run",			//Run Java Application.
					"org.eclipse.equinox.p2.ui.sdk.install",					//Help -> Install New Software.
					"org.eclipse.ui.help.tipsAndTricksAction",					//Help -> Tips and Tricks. Open the tips and tricks help page.
					"org.eclipse.ui.help.quickStartAction",						//Help -> Welcome. Show help for beginning users.
					"org.eclipse.ui.help.helpContents",							//Help -> Help Contents. Open the help contents.
					"org.eclipse.search.ui.openSearchDialog",					//Open the Search dialog.
					"org.eclipse.search.ui.openFileSearchPage",					//Open the Search dialog's file search page.
/*USIKKERT*/		"org.eclipse.jdt.ui.actions.OpenPackageWizard",				//New Java Package. 
					"org.eclipse.jdt.ui.JavaElementCreationActionSet/org.eclipse.jdt.ui.actions.OpenPackageWizard",
					"org.eclipse.jdt.ui.JavaElementCreationActionSet",
					"org.eclipse.jdt.ui.actions.NewTypeDropDown",				//New Java Class.
					"org.eclipse.jdt.ui.JavaHierarchyPerspective",				//Show the Java Type Hierarchy perspective.
					"org.eclipse.ui.edit.text.openLocalFile",					//Open a file.
					"org.eclipse.jdt.ui.JavaPerspective",						//Show the Java perspective.
					"org.eclipse.ui.file.newQuickMenu",							//Open the New menu.
					"org.eclipse.ui.window.showContextMenu",					//Show the context menu.
					"org.eclipse.jdt.ui.edit.text.java.search.occurrences.in.file.quickMenu",//Shows the Occurrences in File quick menu.
					"org.eclipse.jdt.ui.JavaElementCreationActionSet/org.eclipse.jdt.ui.actions.OpenProjectWizard",//New Java Project.
					"org.eclipse.jdt.ui.actions.OpenProjectWizard",
					"org.eclipse.debug.ui.DebugPerspective",					//Open the debug perspective.
					"org.eclipse.ui.window.showViewMenu"						//Show the view menu.

				};
		    String initialSelection = "org.eclipse.ui.newWizard";
		    String selection = (String)JOptionPane.showInputDialog(null, "Choose a command:", "Execute Command", JOptionPane.PLAIN_MESSAGE, null, selectionValues, initialSelection);
		    System.out.println("CommandSelection: "+ selection);			
			
			//Execute a command.
			IHandlerService service = (IHandlerService) getSite().getService(IHandlerService.class);
			eventTextField.append("\n"+Methods.getCurrentTime()+", IHandlerService service: " + service);
			try {
				if( selection != null ){
					if( selection.equals("org.eclipse.jdt.ui.JavaElementCreationActionSet") ){
						//ParameterizedCommand.generateCommand(selection, null);
						Methods.executeCommand(selection);
						
					}else{
						service.executeCommand(selection, null);		
					}
					eventTextField.append("\n"+Methods.getCurrentTime()+",Executing command: " + selection);
				}
				
			} catch (ExecutionException e) {
				System.out.println("\nExecutionException: " + e);
				e.printStackTrace();
			} catch (NotDefinedException e) {
				System.out.println("\nNotDefinedException: " + e);
				e.printStackTrace();
			} catch (NotEnabledException e) {
				System.out.println("\nNotEnabledException: " + e);
				e.printStackTrace();
			} catch (NotHandledException e) {
				System.out.println("\nNotHandledException: " + e);
				e.printStackTrace();
			}
			
//End TESTING
			
		}else{
			appendText("MouseUpEvent(Button " + event.button + "): \nEvent toString: " + event.toString());
		}	
			
	}//handleMouseUp.
	
	/**
	 * Method for handling a Selection Event.
	 * @param event
	 */
	public void handleSelection(Event event) {
		if( !event.widget.toString().contains("ScrollBar") ){	//For  unng  skrive ut ScrollBar events. Dvs trykke p scrollbar eller bevege p den.
			appendText("SelectionEvent: " + event);
		}		
		
		//Sjekke om seleksjonen har vrt av typen MenuItem.
		if( event.widget instanceof MenuItem ){
			MenuItem menuItem = (MenuItem)event.widget;			
//			Methods.menuItemSelectionEventHandler(menuItem, event);	// MenuItem-event-selection-handler.		
		}//menuitem.
		
		//Sjekke om seleksjonen er av typen: ToolItem. Ex: "Run" button on the main-toolbar.
		else if( event.widget instanceof ToolItem ){
			ToolItem toolItem = (ToolItem)event.widget;			
//			Methods.toolItemSelectionEventHandler(toolItem, event);	// ToolItem-event-selection-handler.								
		}
		//Sjekke om seleksjonen er av typen: Tree.	Ex:	Klikke p "MyFirstProject" i view'et "Package Explorer".
		else if( event.widget instanceof Tree ){			
			Tree tree = (Tree)event.widget; // Tree-event-object.			
//			Methods.treeSelectionEventHandler(tree); // Tree-event-selection-handler.
		}//Tree.		
		
		//Sjekke om seleksjonen er av typen: Button. Ex: "Canel", "OK".
		else if( event.widget instanceof Button ){	//event.widget == Button {Cancel}, button.getText() == Cancel, button.getShell().getText() == Preferences.
			Button button = (Button)event.widget;			
//			Methods.buttonSelectionEventHandler(button, event);	// Button-event-selection-handler.		
		}//button.
		
		//Sjekke om seleksjonen har vrt av typen CTabFolder.
		else if( event.widget instanceof CTabFolder ){
			CTabFolder ctabfolder = (CTabFolder)event.widget;			
//			Methods.cTabFolderSelectionEventHandler(ctabfolder, event);	// CTabFolder-event-selection-handler.	
		}//CTabFolder.
		
		//Sjekke om seleksjonen er av typen: ScrollBar.
		else if( event.widget instanceof ScrollBar ){	//event.widget == ScrollBar {}, 
			/*try {
				ScrollBar scrollbar = (ScrollBar)event.widget;
				Scrollable scrollable = scrollbar.getParent();
				eventTextField.append("\nScrollable.getShell.getText: " +scrollable.getShell().getText());
			} catch (Exception e) {
				System.out.println("\n\n"+getCurrentTime()+", Exception(Selection Event ScrollBar): " + e.toString());
				e.printStackTrace();
			}*/
			//System.out.println("\n\n"+getCurrentTime()+", Selection Event ScrollBar: " +event.widget);
		}//scrollbar.
		
	}// end handleSelection.	
	
	/**
	 * Method for getting the Display for this workbench.
	 * @return Display
	 */
	public Display getDisplay() {
		return getSite().getWorkbenchWindow().getWorkbench().getDisplay();
	}	
	
	/** 
	 * Method for appending text to the event text field.
	 */
	public static void appendText(String string){
		if(showEventsInEventTextField){
			eventCounter++;
			String str = "\n\n" + eventCounter + ". ["+Methods.getCurrentTime()+"]: " + string;
			//Printing the event into the text field.
			eventTextField.append(str);
			//Adding the textual representation of the event to the ArrayList. Storing all of the events that has been printed out in the eventtextfield-window.
			eventTextFieldContent.add(str);
		}
	}
	
	/**
	 * Method for clearing a text field. Takes a Text field as a given parameter.
	 * @param text
	 */
	public void clearTextField(Text text){
		text.setText("");
	}
	
	/**
	 * Method for printing the ArrayList "eventTextFieldContent" into the event text field.
	 */
	public void printEventTextFieldContent(){
		for(int i=0; i<eventTextFieldContent.size(); i++) {
			eventTextField.append(eventTextFieldContent.get(i));
		}
	}	
	
	/** 
	 * Method for appending mouse cursor location coordinates into the event text field.
	 */
	public void showMouseCursorLocation() {
		appendText("MouseCursorLocation: x=" + getDisplay().getCursorLocation().x + " y=" + getDisplay().getCursorLocation().y);
	}	
	
	/**
	 * Method for updating the event text field status label.
	 */
	public void updateEventTextFieldStatusLabel(){
		eventTextFieldStatusLabel.setText("Listening after events: [" +showEventsInEventTextField+ "]");
		eventTextFieldStatusLabel.redraw(); //For  unng bug der den siste hakeparentesen ikke blir vist.
	}
		
	/**
	 * Method for returning the current scenario acitivated by the user
	 * @return int, or 0 if a scenario is not activated by the user
	 */
	public int getCurrentScenario(){
		return this.currentScenario;
	}
	
	/**
	 * Method for setting the current scenario activated by the user
	 * @param int
	 */
	public void setCurrentScenario(int scenario){
		this.currentScenario = scenario;
	}	
	
	/** 
	 * Method for setting the focus of the view.
	 */
	@Override
	public void setFocus() {
	}
	
	/**
	 * Method for adding listeners to the workbench.
	 */
	public void addListeners(){
		PlatformUI.getWorkbench().getActiveWorkbenchWindow().getPartService().addPartListener(iPartListener);
		PlatformUI.getWorkbench().getActiveWorkbenchWindow().addPerspectiveListener(perspectiveListener);
		getDisplay().addFilter(SWT.MouseMove, mouseMoveListener);
		getDisplay().addFilter(SWT.MouseUp, mouseUpListener);
		getDisplay().addFilter(SWT.Selection, selectionListener);
		getDisplay().addFilter(SWT.MenuDetect, menuDetectListener);	
		getDisplay().addFilter(SWT.Show, showListener);
		getDisplay().addFilter(SWT.Expand, expandListener);
		getDisplay().addFilter(SWT.Close, closeListener);
		getDisplay().addFilter(SWT.KeyUp, keyUpListener);
		getDisplay().addFilter(SWT.Arm, new Listener() {			
			@Override
			public void handleEvent(Event event) {
				System.out.println("Arm-Event: " + event+ ", keycode: "+event.keyCode+", button:"+event.button+", item: "+event.item+", getBounds: "+event.getBounds()+", getData: "+event.widget.getData());
				if( event.widget instanceof MenuItem ){
					MenuItem menuitem = (MenuItem)event.widget;
					System.out.println("MenuItem: " +menuitem+ ", selection: " + menuitem.getSelection());
					if( menuitem.getMenu() != null ){
						System.out.println("MenuItem-Menu: " + menuitem.getMenu()+", items: "+menuitem.getMenu().getItemCount());
					}
				}
				else if( event.widget instanceof Menu ){
					Menu menu = (Menu)event.widget;
					System.out.println("Menu: " +menu+ ", items: " + menu.getItemCount()+", visible: "+menu.getVisible()+", defaultItem: "+menu.getDefaultItem());
					System.out.println("Menu: " + menu+", getData: "+menu.getData()  );
				}
				System.out.println();
			}
		});
		
	
		
		
//TODO 	Testing
		//IExecutionListener
		iexelistener = new IExecutionListener() {				
			@Override
			public void preExecute(String commandId, ExecutionEvent event) {
				appendText(Methods.getCurrentTime()+", ExecutionEvent: " + event);
				eventTextField.append("\nExecutionEvent: "+event);
				eventTextField.append("\nCommandID[eventGetCommandID]: "+event.getCommand().getId() );
				eventTextField.append("\nCommandID: " + commandId);
				try {
					eventTextField.append("\nCommandName: "+event.getCommand().getName() );
					eventTextField.append("\nCommandDescription: "+event.getCommand().getDescription() );
				} catch (NotDefinedException e) {
					System.out.println("NotDefinedException: " + e);
					e.printStackTrace();
				}
				
				//Sjekke om event hrer til et scenario.
				try {
					//System.out.println("event.getTrigger(): " + event.getTrigger());
					if( event.getTrigger() instanceof StyledText ){
						StyledText styledtext = (StyledText)event.getTrigger();
						ScenarioHandler.scenarioVerify(styledtext);
						System.out.println(event.getTrigger() + ", instance of StyledText");
					}
					else if( event.getTrigger() == null ){	//could be a programmatically executed command. 
						//System.out.println("event.getTrigger(), is null. Command: " + event.getCommand().getName() );
					}
				} catch (Exception e) {
					System.out.println("[MouseEventListenerView]: IExecutionListener exception: " + e);
					e.printStackTrace();
				}				
			}			
			@Override
			public void postExecuteSuccess(String commandId, Object returnValue) {
			}			
			@Override
			public void postExecuteFailure(String commandId,ExecutionException exception) {					
			}			
			@Override
			public void notHandled(String commandId, NotHandledException exception) {					
			}
		};
		
		commandService = (ICommandService) PlatformUI.getWorkbench().getActiveWorkbenchWindow().getService(ICommandService.class);
		commandService.addExecutionListener(iexelistener);	//adding executionlistener.
		
	}//add listeners.
	
	/**
	 * Method for releasing any resources. Any added listener will be removed by this method.
	 */
	@Override
	public void dispose() {
		System.out.println("\n\n" +Methods.getCurrentTime()+ ": Disposing the MouseEventListenerView.java");
		//Removing listeners.
		PlatformUI.getWorkbench().getActiveWorkbenchWindow().getPartService().removePartListener(iPartListener);
		PlatformUI.getWorkbench().getActiveWorkbenchWindow().removePerspectiveListener(perspectiveListener);
		getDisplay().removeFilter(SWT.MouseMove, mouseMoveListener);
		getDisplay().removeFilter(SWT.MouseUp, mouseUpListener);
		getDisplay().removeFilter(SWT.Selection, selectionListener);
		getDisplay().removeFilter(SWT.MenuDetect, menuDetectListener);	
		getDisplay().removeFilter(SWT.Show, showListener);
		getDisplay().removeFilter(SWT.Expand, expandListener);
		getDisplay().removeFilter(SWT.Close, closeListener);
		getDisplay().removeFilter(SWT.KeyUp, keyUpListener);
		
		commandService.removeExecutionListener(iexelistener);	//removing executionlistener.
		
		getSite().getWorkbenchWindow().getSelectionService().removeSelectionListener(iSelectionListener);//removing iselectionlistener.

	}// dispose.

}// end MouseEventListenerView.java