package com.eclipse.listener.views;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;

import javax.swing.UIManager;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.part.ViewPart;

import com.eclipse.listener.helpClasses.Methods;
import com.eclipse.listener.scenario.Scenario;
import com.eclipse.listener.scenario.ScenarioProgressListener;
import com.eclipse.listener.scenario.task.CommandActivate;
import com.eclipse.listener.scenario.task.ContextMenuItemClick;
import com.eclipse.listener.scenario.task.DialogButtonClick;
import com.eclipse.listener.scenario.task.KeyboardTextType;
import com.eclipse.listener.scenario.task.MenuBarClick;
import com.eclipse.listener.scenario.task.MenuBarItemClick;
import com.eclipse.listener.scenario.task.OpenJavaProjectContextMenu;
import com.eclipse.listener.scenario.task.Task;
import com.eclipse.listener.scenario.task.ToolBarClick;
import com.eclipse.listener.scenario.task.TreeItemClick;
import com.eclipse.listener.scenario.task.TreeItemClickIsAJavaProject;
import com.eclipse.listener.scenario.task.WorkbenchPartActivate;
import com.eclipse.listener.scenario.task.autocomplete.ClickDialogButtonInNewWizardExecution;
import com.eclipse.listener.scenario.task.autocomplete.ContextMenuItemClickExecution;
import com.eclipse.listener.scenario.task.autocomplete.EclipseCommandExecution;
import com.eclipse.listener.scenario.task.autocomplete.JavaProjectInPackageExplorerExecution;
import com.eclipse.listener.scenario.task.autocomplete.MenuBarClickExecution;
import com.eclipse.listener.scenario.task.autocomplete.MenuBarItemClickExecution;
import com.eclipse.listener.scenario.task.autocomplete.OpenAJavaFileExecution;
import com.eclipse.listener.scenario.task.autocomplete.PasteTextInAJavaFileExecution;
import com.eclipse.listener.scenario.task.autocomplete.SelectTreeItemInNewWizardExecution;
import com.eclipse.listener.scenario.task.autocomplete.ToolBarClickExecution;
import com.eclipse.listener.scenario.task.autocomplete.TreeItemClickExecution;
import com.eclipse.listener.scenario.task.autocomplete.TreeItemClickExecution2;

public class ScenarioView extends ViewPart implements ScenarioProgressListener {
	//Variables.
	private Collection<Scenario> scenarios = new ArrayList<Scenario>();			// A collection of the Scenario-objects.
	private Collection<Composite> taskComposites = new ArrayList<Composite>();	// A collection of the Task-Composite for every Scenario.
	private Scenario currentScenarioActivated = null;							// Containing the currently activated Scenario, or Null if no Scenario is activated.
	
	private Label scenarioLabel;												
	private ScrolledComposite scrollableScenarioField;							
	private Composite scenarioField;
	private Label achievementLabel;												
	private Text achievementTextField;											
	
	@Override
	public void createPartControl(Composite parent) {
		// View Layout: the layout for the parent-Composite.
		int columnWidth = 1;
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = columnWidth;
		parent.setLayout(gridLayout);
		
		// Set the look and feel to the users system look and feel!!!
		try {
			System.out.println("Current Platform OS: " + org.eclipse.core.runtime.Platform.getOS() );
			System.out.println("Current Platform OS Architechture: " + org.eclipse.core.runtime.Platform.getOSArch() );
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			System.out.println("[ScenarioView]: getSystemLookAndFeelClassName, success");
		} catch (Exception e) {
			System.out.println("\n\n"+Methods.getCurrentTime()+", Exception(UIManager look and feel): " + e.toString());
			e.printStackTrace();
		}			
		
		//Scenario label.
		scenarioLabel = new Label(parent, SWT.NONE);
		scenarioLabel.setText("Scenario:");
		GridData gridData = new GridData(SWT.BEGINNING, SWT.BEGINNING, false, false);
		gridData.horizontalSpan = columnWidth;
		scenarioLabel.setLayoutData(gridData);
		
		//ScrollableScenarioField - Composite.
		scrollableScenarioField = new ScrolledComposite(parent, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
		scrollableScenarioField.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		scrollableScenarioField.setAlwaysShowScrollBars(true);
		scrollableScenarioField.setExpandHorizontal(true);
		scrollableScenarioField.setExpandVertical(true);
		//Enabling mouse-wheel-scrolling in the scrollable composite.
		scrollableScenarioField.addListener(SWT.MouseVerticalWheel, new Listener() {
            public void handleEvent(Event event) {
            	//System.out.println("MouseVerticalWheel[scrollableScenarioField]: " + event);
                int wheelCount = event.count;
                wheelCount = (int) Math.ceil(wheelCount / 3.0f);
                while (wheelCount < 0) {
                	scrollableScenarioField.getVerticalBar().setIncrement(4);
                    wheelCount++;
                }

                while (wheelCount > 0) {
                	scrollableScenarioField.getVerticalBar().setIncrement(-4);
                    wheelCount--;
                }
            }
        });		
		
		// Scenario composite field.
		scenarioField = new Composite(scrollableScenarioField, SWT.BORDER);
		gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
		scenarioField.setLayoutData(gridData);
		createScenariosAndTasks(); //Creating the Scenarios and their Tasks.
		//Adding ScenarioView as a listener to every Scenario.
		for(Scenario scenario : scenarios) {
			scenario.addScenarioProgressListener(this);
		}
		initScenarioIntoScenarioField(scenarioField);
		//Setting focus to the scrollable composite whenever there is a mouse up event inside of it.
		scenarioField.addListener(SWT.MouseUp, new Listener() {			
			@Override
			public void handleEvent(Event event) {
				//System.out.println("MouseUp[scrollableScenarioField]: " + event);
				scrollableScenarioField.setFocus();
			}
		});		
		
		scrollableScenarioField.setContent(scenarioField);
		scrollableScenarioField.setMinSize(scenarioField.computeSize(SWT.DEFAULT, SWT.DEFAULT)); //Denne m vre med for at det skal vre mulig  scrolle vannrett og loddrett nr vinduet blir mindre.
//		scenarioField.redraw();
		

// OBS! Dette tekstvinduet er fjernet da det ikke er implementert noen form for belnning i det en task/scenario fullfres!		
//		//Achievements label.
//		achievementLabel = new Label(parent, SWT.NONE);
//		achievementLabel.setText("Achievements: 0");
//		gridData = new GridData(SWT.BEGINNING, SWT.BEGINNING, false, false);
//		gridData.horizontalSpan = columnWidth;
//		achievementLabel.setLayoutData(gridData);
//		
//		// Achievement text field.
//		achievementTextField = new Text(parent, SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL);
//		achievementTextField.setEditable(false);
//		gridData = new GridData(SWT.FILL, SWT.BOTTOM, true, false);
//		gridData.horizontalSpan = columnWidth;
//		gridData.heightHint = 100;
//		achievementTextField.setLayoutData(gridData);		
		
		scenarioField.pack();
		
	}//end createPartControl.		

	/**
	 * Method for creating the different Scenarios, their Tasks and their AutoExecution-objects.
	 */
	private void createScenariosAndTasks() {		
		Scenario scenario1 = new Scenario("Scenario 1 - Create A New Java Project");
		scenario1.addTask(new ToolBarClick("New", "1. Click on the 'New'-button in the toolbar.", new ToolBarClickExecution("New")));
		scenario1.addTask(new TreeItemClick("Java Project", "2. Select 'Java Project'.", new SelectTreeItemInNewWizardExecution("Java Project")));
		scenario1.addTask(new DialogButtonClick("Next", "3. Click on the 'Next'-button.", new ClickDialogButtonInNewWizardExecution("Next")));
		scenario1.addTask(new DialogButtonClick("Finish", "4. Enter a project name, then click on the 'Finish'-button.", new ClickDialogButtonInNewWizardExecution("Finish")));
		
		Scenario scenario2 = new Scenario("Scenario 2 - Open Debug Perspective");
		scenario2.addTask(new MenuBarClick("Window", "1. Click on the 'Window'-button in the menubar.", new MenuBarClickExecution("Window")));
		scenario2.addTask(new MenuBarItemClick("Open Perspective", "2. Click on 'Open Perspective'.", new MenuBarItemClickExecution("Open Perspective")));
		scenario2.addTask(new MenuBarItemClick("Debug", "3. Click on 'Debug'.", new MenuBarItemClickExecution("Debug")));
		
		Scenario scenario3 = new Scenario("Scenario 3 - Open Console View");
		scenario3.addTask(new MenuBarClick("Window", "1. Click on the 'Window'-button in the menubar.", new MenuBarClickExecution("Window")));
		scenario3.addTask(new MenuBarItemClick("Show View", "2. Click on 'Show View'.", new MenuBarItemClickExecution("Show View")));
		scenario3.addTask(new MenuBarItemClick("Console", "3. Click on 'Console'.", new MenuBarItemClickExecution("Console")));
		
		Scenario scenario4 = new Scenario("Scenario 4 - Open Java Build Path");
		scenario4.addTask(new TreeItemClickIsAJavaProject("Random Java Project", "1. Select a Java Project in Package Explorer.", new JavaProjectInPackageExplorerExecution()));
		scenario4.addTask(new OpenJavaProjectContextMenu("Context Menu", "2. Right-click on the selected java project.", new EclipseCommandExecution("Context Menu Java Project")));
		scenario4.addTask(new ContextMenuItemClick("Properties", "3. Click on 'Properties'.", new ContextMenuItemClickExecution("Properties")));
		scenario4.addTask(new TreeItemClick("Java Build Path", "4. Click on 'Java Build Path'.", new TreeItemClickExecution2("Java Build Path")));
		
		Scenario scenario5 = new Scenario("Scenario 5 - Open Preferences");
		scenario5.addTask(new MenuBarClick("Window", "1. Click on the 'Window'-button in the menubar.", new MenuBarClickExecution("Window")));
		scenario5.addTask(new MenuBarItemClick("Preferences", "2. Click on 'Preferences'.", new MenuBarItemClickExecution("Preferences")));
		
		Scenario scenario6 = new Scenario("Scenario 6 - Use Content Assist");
		scenario6.addTask(new WorkbenchPartActivate("Java File", "1. Open a Java File.", new OpenAJavaFileExecution()));
		scenario6.addTask(new KeyboardTextType("Sysout", "2. Type 'sysout' inside a method.", new PasteTextInAJavaFileExecution("sysout")));
		scenario6.addTask(new CommandActivate("Content Assist", "3. Click CTRL + SPACE on the keyboard.", SWT.KeyDown, new EclipseCommandExecution("Content Assist")));		
		
		//Adding every Scenario-object to the collection of scenarios.
		scenarios.addAll(Arrays.asList(scenario1, scenario2, scenario3, scenario4, scenario5, scenario6));		
	}//end createScenariosAndTasks.

	/**
	 * Method for adding the Scenario and their Tasks to composite.
	 */
	public void initScenarioIntoScenarioField(Composite parent){		
		// View Layout - the layout for the parent-Composite.
		int parentColumnWidth = 1;	// The number of elements in each row in the parent-Composite.
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = parentColumnWidth;
		gridLayout.marginTop = 10;
		parent.setLayout(gridLayout);
		
		//Scenario-title-buttons standard font. Will be used for every Scenario Title Buttons.
		Font scenarioTitleButtonsFont = new Font( parent.getDisplay(), "Arial", 11, SWT.BOLD ); 	
			
		//Iterate through Scenarios and creating GUI-components for Scenario-Button.	
		for(final Scenario scenario : scenarios) {			
			final Button title = new Button(parent, SWT.NONE);
			title.setText( scenario.getTitle() );
			title.setFont( scenarioTitleButtonsFont );
			title.setToolTipText("Push to activate or deactivate scenario");
			GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, false);
			title.setLayoutData(gridData);
			
			final Composite tasksComposite = new Composite(parent, SWT.BORDER);
			//Setting focus to the scrollable composite whenever there is a mouse up event inside of the tasks-Composite.
			tasksComposite.addListener(SWT.MouseUp, new Listener() {			
				@Override
				public void handleEvent(Event event) {
					scrollableScenarioField.setFocus();
				}
			});
			gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
			gridData.exclude = true;
			tasksComposite.setLayoutData( gridData );
			int tasksColumnWidth = 3;		// The number of elements in each row in the tasks-Composite.
			gridLayout = new GridLayout();	// Layout for the tasks-Composite.
			gridLayout.numColumns = tasksColumnWidth;
			tasksComposite.setLayout(gridLayout);
			
			title.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					//Activate Scenario.
					activateScenario(scenario, tasksComposite, title);
				}
			});
			
			//Iterate through the Tasks of the Scenario and creating GUI-components: Help-Button, Check-Box and Label.
			for(final Task task : scenario.getTasks()) {
				Button helpButton = new Button(tasksComposite, SWT.NONE);
				helpButton.setText("Help");
				helpButton.setToolTipText("Press for autocompleting the task.");
				gridData = new GridData(SWT.FILL, SWT.CENTER, false, false);
				helpButton.setLayoutData( gridData );
				helpButton.addSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(SelectionEvent e) {
						System.out.println("\nHelp button pressed: " + task.getTitle());
						task.autoExecuteTask();
					}
				});					
				Button checkboxButton = new Button(tasksComposite, SWT.CHECK);
				checkboxButton.setEnabled(false);
				gridData = new GridData(SWT.FILL, SWT.CENTER, false, false);
				checkboxButton.setLayoutData( gridData );
				Label taskLabel = new Label(tasksComposite, SWT.NONE);
				taskLabel.setText(task.getDescription());
				gridData = new GridData(SWT.FILL, SWT.CENTER, true, false);
				taskLabel.setLayoutData( gridData );				
			}//foreach Task.
			
			//Adding the tasksComposite to the collection of Task-Composites.
			taskComposites.add(tasksComposite);
		}//foreach Scenario.

	}//initScenarioIntoScenarioField.
		
	/**
	 * Method for activating a Scenario whenever a Scenario-title-button is clicked.
	 * @param scenario Scenario: The Scenario of the clicked Scenario-button.
	 * @param tasksComposite Composite: The Tasks-Composite of the clicked Scenario-button.
	 */
	public void activateScenario(Scenario scenario, Composite tasksComposite, Button ScenarioButton) {
		System.out.println("\n"+Methods.getCurrentTime()+", activateScenario():\nScenario:"+scenario.toString()+"\nTasksComposite:");
		for(Control control : tasksComposite.getChildren()) {
			System.out.println(control.toString());
		}
		
		//There is currently NOT activated any Scenario.
		if( currentScenarioActivated == null ){
			System.out.println("No previously activated Scenario, therefore activating Scenario: " + scenario.getTitle());
			currentScenarioActivated = scenario;			
			//Activate the Scenario-logic only if the Scenario has not been finished yet.
			if( !currentScenarioActivated.isFinished() ){
				colorizeControl(ScenarioButton, SWT.COLOR_BLUE);
				currentScenarioActivated.activateScenario();
			}
			//Scenario is already finished, give info message to user.
			else{
				//JOptionPane.showMessageDialog(null, "Scenario: "+currentScenarioActivated.getTitle()+" is already finished!");
				MessageDialog.openInformation(null, "Scenario Completed", "Scenario: "+currentScenarioActivated.getTitle()+" is already finished!");
			}
			showComposite(tasksComposite);	//Show the Tasks-Composite.
		}
		//There is already a currently activated Scenario.
		else{
			//Clicking the Scenario-button of the currently activated Scenario -> Close/deactivate the currently activated Scenario.
			if( currentScenarioActivated.getTitle().equals(scenario.getTitle()) ){
				System.out.println("Currently activated Scenario: " +currentScenarioActivated.getTitle()+ ", Clicked Scenario: " +scenario.getTitle() );
				boolean result = MessageDialog.openQuestion(null, "Exit Scenario", "Do you want to exit the Scenario?");
				if(result){	//Yes-Button.
					//Exit Scenario.
					System.out.println("EXITING currently activated Scenario: " + currentScenarioActivated.getTitle());
					currentScenarioActivated.deactivateScenario();	//Deactivate logic for Scenario and its Tasks.					
					hideComposite(tasksComposite);					//Hide the Tasks-Composite.
					colorizeScenarioButton(ScenarioButton); 		//Coloring the Scenario-button: Red if it is not finished.
					currentScenarioActivated = null;				//Reset currently activated Scenario.
				}
			}
			//Clicking the Scenario-button of a Scenario while a different Scenario is currently activated -> Give error message to the user.
			else{
				System.out.println("Currently activated Scenario: " +currentScenarioActivated.getTitle()+ ", Clicked Scenario: " +scenario.getTitle() );
				MessageDialog.openError(null, "Error!", "There is already activated a Scenario:\n\n"+currentScenarioActivated.getTitle()+"\n\nTo start another Scenario please finish or exit this Scenario first.");
			}
		}		
	}//end activateScenario.
	
	/**
	 * Method for showing a Composite: <br><br><i>setVisible(true), gridData.exclude=false, getParent().pack()</i>
	 * @param composite - Composite to be shown.
	 */
	public void showComposite(Composite composite){
		composite.setVisible(true);
		GridData gd = (GridData)composite.getLayoutData();
		gd.exclude = false;
		if( composite.getParent() != null ){
			System.out.println("composite.getParent().pack() -> show");
			composite.getParent().pack();
		}else{
			System.out.println("composite.pack() -> show");
			composite.pack();
		}
	}//end showComposite.
	
	/**
	 * Method for hiding a Composite: <br><br><i>setVisible(false), gridData.exclude=true, getParent().pack()</i>
	 * @param composite - Composite to be shown.
	 */
	public void hideComposite(Composite composite){
		composite.setVisible(false);
		GridData gd = (GridData)composite.getLayoutData();
		gd.exclude = true;
		if( composite.getParent() != null ){
			System.out.println("composite.getParent().pack() -> hide");
			composite.getParent().pack();
		}else{
			System.out.println("composite.pack() -> hide");
			composite.pack();
		}
	}//end hideComposite.	

	/**
	 * Method for getting the Display for this workbench.
	 * @return Display
	 */
	public Display getDisplay() {
		return getSite().getWorkbenchWindow().getWorkbench().getDisplay();
	}
	
	/**
	 * Method for coloring a Scenario-button. Green when it is finished, or red when it is not finished and gets deactivated.
	 * @param scenarioButton Button
	 */
	public void colorizeScenarioButton(Button scenarioButton){
		Color color = null;
		//The Scenario is finished.
		if( currentScenarioActivated != null && currentScenarioActivated.isFinished() ){
			//Make sure that scenarioButton is not already colored GREEN.
			if( scenarioButton.getBackground() != Display.getCurrent().getSystemColor(SWT.COLOR_GREEN) ){
				color = Display.getCurrent().getSystemColor(SWT.COLOR_GREEN);
			}			
		}
		//The Scenario is not finished.
		else{
			color = Display.getCurrent().getSystemColor(SWT.COLOR_RED);
		}
		//Coloring.
		if( color != null ){
			scenarioButton.setBackground( color );	//P Windows-operativsystemer kan man kun fargelegge kantlinjen rundt knappen.
			//Disposing resource.
			color.dispose();
			color = null;
		}		
	}//colorizeScenarioButton.
	
	/**
	 * Method for coloring a Control-object.
	 * @param control Control 	- Ex: Button
	 * @param swtColor Integer 	- Ex: SWT.COLOR_GREEN, SWT.COLOR_RED.
	 */
	public void colorizeControl(Control control, int swtColor){
		if( control == null ){
			System.out.println("Error: colorizeControl(), parameter control is null!");
			return;
		}
		Color color = null;
		//Make sure that the Control is not already colored with the parameter color..
		if( control.getBackground() != Display.getCurrent().getSystemColor( swtColor ) ){
			color = Display.getCurrent().getSystemColor( swtColor );
		}
		//Coloring.
		if( color != null ){
			control.setBackground( color );	//P Windows-operativsystemer kan man kun fargelegge kantlinjen rundt knappen.
			//Disposing resource.
			color.dispose();
			color = null;
		}		
	}//end colorizeControl.

	@Override
	public void setFocus() {
	}
	
	@Override
	public void dispose() {
		System.out.println("\n"+Methods.getCurrentTime()+", Dispose: "+getClass().getSimpleName());
		//Deactivate any currently activated Scenario including its Tasks and Listeners.
		if( currentScenarioActivated != null){
			currentScenarioActivated.deactivateScenario();
		}
	}

	@Override
	public void taskActivated(Task task) {
		System.out.println("[ScenarioView]: taskActivated: " + task);
		Composite taskComposite = findTaskComposite(task);					//Given a Task, find its Tasks-Composite.
		int taskIndex = findTaskIndex(task);								//Given a Task, find its index in the Tasks-Composite.		
		if( taskComposite != null ){
			Control[] taskCompositeChildren = taskComposite.getChildren();	//Get Control objects inside Tasks-Composite.
			System.out.println("taskIndex: " + taskIndex + ", compositeChildren size: " + taskCompositeChildren.length);
			int currentRowIndex = 0;										//Keeps track of the current row in the Tasks-Composite.
			for(Control control : taskCompositeChildren) {					//Iterate through Controls.
				System.out.println("Control: " +control+ ", rowIndex: " + currentRowIndex);
				//Button.
				if( control instanceof Button ){
					Button button = (Button)control;
					//Help-Button. This is the leftmost component in the Tasks-Composite.
					if( button.getText().equals("Help") ){
						//Only operate on the composite-row of the given task.
						if( currentRowIndex == taskIndex ){
							colorizeControl(button, SWT.COLOR_BLUE);
							System.out.println("Colorize Activated Help-Button");
						}						
					}
					//Checkbox-Button. This is the middle component in the Tasks-Composite.
					else{
						//Only operate on the composite-row of the given task.
						if( currentRowIndex == taskIndex ){
//							colorizeControl(button, SWT.COLOR_WHITE);
							System.out.println("Colorize Activated Checkbox-Button");
						}
					}					
				}
				//Label. This is the rightmost component in the Tasks-Composite.
				else if( control instanceof Label ){
					//Only operate on the composite-row of the given task.
					if( currentRowIndex == taskIndex ){
//						colorizeControl(control, SWT.COLOR_WHITE);
						System.out.println("Colorize Activated Label");
					}
					currentRowIndex++;		//Increment current row index.
					System.out.println("New row");
					//Do not want to operate on latter rows of the task.
					if( currentRowIndex > taskIndex ){
						break;
					}
				}
			}
		}
	}//end taskActivated.

	@Override
	public void taskCompleted(Task task) {
		System.out.println("[ScenarioView]: taskCompleted: " + task);
		Composite taskComposite = findTaskComposite(task);					//Given a Task, find its Tasks-Composite.
		int taskIndex = findTaskIndex(task);								//Given a Task, find its index in the Tasks-Composite.		
		if( taskComposite != null ){
			Control[] taskCompositeChildren = taskComposite.getChildren();	//Get Control objects inside Tasks-Composite.
			System.out.println("taskIndex: " + taskIndex + ", compositeChildren size: " + taskCompositeChildren.length);
			int currentRowIndex = 0;										//Keeps track of the current row in the Tasks-Composite.
			for(Control control : taskCompositeChildren) {					//Iterate through Controls.
				System.out.println("Control: " +control+ ", rowIndex: " + currentRowIndex);
				//Button.
				if( control instanceof Button ){
					Button button = (Button)control;
					//Help-Button. This is the leftmost component in the Tasks-Composite.
					if( button.getText().equals("Help") ){
						//Only operate on the composite-row of the given task.
						if( currentRowIndex == taskIndex ){
							colorizeControl(button, SWT.COLOR_GREEN);
							button.setEnabled(false);
							System.out.println("Diable Help-Button");
						}						
					}
					//Checkbox-Button. This is the middle component in the Tasks-Composite.
					else{
						//Only operate on the composite-row of the given task.
						if( currentRowIndex == taskIndex ){
							//colorizeControl(button, SWT.COLOR_GREEN);
							button.setSelection(true);
							System.out.println("Check Checkbox-Button");
						}
					}					
				}
				//Label. This is the rightmost component in the Tasks-Composite.
				else if( control instanceof Label ){
					//Only operate on the composite-row of the given task.
					if( currentRowIndex == taskIndex ){
						colorizeControl(control, SWT.COLOR_GREEN);
						System.out.println("Colorize Completed Label");
					}
					
					currentRowIndex++;		//Increment current row index.
					System.out.println("New row");
					//Do not want to operate on latter rows of the task.
					if( currentRowIndex > taskIndex ){
						break;
					}
				}
			}
		}
	}//end taskCompleted.
	
	@Override
	public void scenarioCompleted(Scenario scenario) {
		System.out.println("[ScenarioView]: scenarioCompleted: " + scenario.getTitle());
		final Button scenarioButton = findScenarioButton(scenario);
		if( scenarioButton != null ){
			colorizeScenarioButton(scenarioButton);
			MessageDialog.openInformation(null, "Congratulations", "You have completed a Scenario!\n\n"+currentScenarioActivated.getTitle());
		}		
	}//end scenarioCompleted.
	
	/**
	 * Method for finding the related taskComposite for the given Task.
	 * @return Composite, or null if the Task does not exist in the collection of Tasks-Composite.
	 */
	private Composite findTaskComposite(Task task){
		//Find the index of the Scenario to the Task given as parameter.
		int index = 0;
		for(Scenario scenario : scenarios) {
			if( scenario == task.getParentScenario() ){
				Object[] composites =  taskComposites.toArray();
				return (Composite)composites[index];
			}
			index++;
		}		
		return null;
	}//end findTaskComposite.
	
	/**
	 * Method that finds the index of a Task in its Scenarios collection of tasks.
	 * @param task Task
	 * @return Integer: index, or -1 if the Task is not connected to a Scenario.
	 */
	private int findTaskIndex(Task task){
		int index = 0;
		for(Task scenarioTask: task.getParentScenario().getTasks()) {	//Find Scenario of the given Task.
			if( scenarioTask.getTitle().equals(task.getTitle()) ){		//Find index of given task from collection of Scenario-tasks.
				return index;
			}
			index++;
		}		
		return -1;
	}//end findTaskIndex.
	
	/**
	 * Method that finds the Scenario-Button related to the given Scenario.
	 * <br>The method finds the taskComposite for a Task in the Scenario,
	 * <br>then finds the parentComposite and iterate through its Control-objects,
	 * <br>looking for a Button where the text matches the Scenarios title.
	 * @param scenario Scenario
	 * @return Button, or null if the Scenario does not have a Scenario-Button.
	 */
	private Button findScenarioButton(Scenario scenario){
		if( scenario.getTasks().isEmpty() ){									 //Need to have at least one Task so we can find its Tasks-Composite.
			return null;
		}
		Composite taskComposite = findTaskComposite(scenario.getTasks().get(0)); //Find the taskComposite for a Task in the Scenario.
		if(taskComposite != null){
			Composite scenarioComposite = taskComposite.getParent();			 //Find the parentComposite for the taskComposite.
			if (scenarioComposite != null) {
				for(Control control : scenarioComposite.getChildren()) {		 //Iterate through Control-objects inside Composite.
					if( control instanceof Button ){
						Button button = (Button)control;
						if( button.getText().equals(scenario.getTitle()) ){		 //Find Button with Scenario-Title-Text.
							return button;
						}
					}
				}
			}
		}
		return null;
	}//end findScenarioButton.

}// end AchievementsView.